@@ -32,15 +32,13 @@ from types import StringType
 from random import randint
 import common
 
-
 class CFGrammar(object):
     """ Class for context-free grammars
 
      :var Rules: grammar rules
      :var Terminals: terminals symbols
      :var Nonterminals: nonterminals symbols
-     :var Start: start symbol
-     :type Start: string
+     :var str Start: start symbol
      :var ntr: dictionary of rules for each nonterminal"""
 
     def __init__(self, gram):
@@ -54,8 +52,9 @@ class CFGrammar(object):
         self.Nonterminals = {r[0] for r in self.Rules}
         self.Terminals = set()
         for r in self.Rules:
-            if type(r[1]) is StringType and r[1] not in self.Nonterminals:
-                self.Terminals.add(r[1])
+            if type(r[1]) is StringType:
+                if r[1] not in self.Nonterminals:
+                    self.Terminals.add(r[1])
             else:
                 for s in r[1]:
                     if s not in self.Nonterminals:
@@ -73,12 +72,14 @@ class CFGrammar(object):
     def __str__(self):
         """Grammar rules
 
-            :return: a string representing the grammar rules"""
+        :return: a string representing the grammar rules"""
         s = ""
         for n in xrange(len(self.Rules)):
             lhs = self.Rules[n][0]
             rhs = self.Rules[n][1]
-            s += "%s | %s -> %s \n" % (n, lhs, string.join(rhs))
+            if type(rhs) is not StringType and len(rhs) > 1:
+                rhs = string.join(rhs)
+            s += "{0:d} | {1:s} -> {2:s} \n".format(n, lhs, rhs)
         return "Grammar Rules:\n\n%s" % s
 
     def maketerminals(self):
@@ -99,6 +100,7 @@ class CFGrammar(object):
             self.Nonterminals.add(r[0])
 
     def terminalrules(self):
+        self.tr = {}
         for a in self.Terminals:
             for i in xrange(len(self.Rules)):
                 if self.Rules[i][1] == a:
@@ -108,6 +110,7 @@ class CFGrammar(object):
                         self.tr[a].add(i)
 
     def nonterminalrules(self):
+        self.ntr = {}
         for i in xrange(len(self.Rules)):
             if self.Rules[i][0] not in self.ntr:
                 self.ntr[self.Rules[i][0]] = {i}
@@ -189,7 +192,7 @@ class CNF(CFGrammar):
 
     def iter_rule(self, lhs, rhs, i):
         if type(rhs) is not StringType and len(rhs) == 2:
-            self.Rules[i] = ((lhs, rhs))
+            self.Rules[i] = (lhs, rhs)
             return
         nta = self.mark + "_" + str(self.newnt)
         self.Nonterminals.add(nta)
@@ -228,8 +231,8 @@ class cfgGenerator(object):
             :type size: integer"""
         self.grammar = cfgr
         self.size = size
-        self.density = {}
-        self.density_r = {}
+ #       self.density = {}
+ #       self.density_r = {}
         self._eval_densities(size)
 
     def generate(self):
@@ -242,12 +245,10 @@ class cfgGenerator(object):
     def _gen(self, nt, n):
         """Generates a new random object generated from the nonterminal
 
-            :param nt: nonterminal
-            :type nt: string
-            :param n: object size
-            :type n: integer
-            :returns: object
-            :rtype: string"""
+        :param str nt: nonterminal
+        :param int n: object size
+        :returns: object
+        :rtype: str"""
         g = self.grammar
         if n in self.density[nt] and self.density[nt][n] > 0:
             u = randint(1, self.density[nt][n])
@@ -285,8 +286,7 @@ class cfgGenerator(object):
     def _eval_densities(self, n):
         """Evaluates densities
 
-            :param n: object size
-            :type n: integer"""
+        :param int n: object size"""
         g = self.grammar
         self.density = {}
         self.density_r = {}
@@ -317,54 +317,49 @@ class cfgGenerator(object):
 class reStringRGenerator(cfgGenerator):
     """Uniform random Generator for reStrings"""
 
-    def __init__(self, Sigma=["a", "b"], size=10, cfgr=None, epsilon=None, empty=None, ident="Ti"):
+    def __init__(self, Sigma=None, size=10, cfgr=None, epsilon=None, empty=None, ident="Ti"):
         """ Uniform random generator for regular expressions. Used without arguments generates an uncollapsible re
                 over {a,b} with size 10. For generate an arbitary re over an alphabet of 10 symbols of size 100:
-                reStringRGenerator (small_alphabet(10),100,reStringRGenerator.g_regular_base)
+                reStringRGenerator (smallAlphabet(10),100,reGrammar["g_regular_base"])
 
-                :param Sigma: re alphabet (that will be the set of grammar terminals)
-                :type Sigma: list or set
-                :param size: word size
-                :type size: integer
-                :param cfgr: base grammar
-                :param epsilon: if not None is added to a grammar terminals
-                :param empty: if not None is added to a grammar terminals
+        :param list|set Sigma: re alphabet (that will be the set of grammar terminals)
+        :param int size: word size
+        :param cfgr: base grammar
+        :param epsilon: if not None is added to a grammar terminals
+        :param empty: if not None is added to a grammar terminals
 
-                .. note::
-                   the grammar can have already this symbols"""
+        .. note::
+           the grammar can have already this symbols"""
         if cfgr is None:
             self.base = gRules(reGrammar["g_regular_uncollaps"])
         else:
             self.base = gRules(cfgr)
-        self.Sigma = Sigma
+        if Sigma is None:
+            self.Sigma = ["a", "b"]
+        else:
+            self.Sigma = Sigma
         for i in self.Sigma:
             self.base.append((ident, i))
         if epsilon is not None:
             self.base.append((ident, common.Epsilon))
         if empty is not None:
             self.base.append((ident, common.EmptySet))
-        self.gen = cfgGenerator(CNF(self.base), size)
-
-    def generate(self):
-        """Generates a new random RE string"""
-        return self.gen.generate()
+        super(reStringRGenerator, self).__init__(CNF(self.base),size)
 
 
 #noinspection PyUnusedLocal
 def CYKParserTable(gramm, word):
     """Evaluates CYK parser table
 
-      :param gramm: grammar
-      :type gramm: CNF
-      :param word: word to be parsed
-      :type word: string
+      :param CNF gramm: grammar
+      :param str word: word to be parsed
       :returns: the CYK table
       :rtype: list of lists of symbols"""
     pass
 
 
 def gRules(rules_list, rulesym="->", rhssep=None, rulesep='|'):
-    """Transforms a list of rules in a grammar description.
+    """Transforms a list of rules into  a grammar description.
 
       :param rules_list: is a list of rule where rule is a string  of the form: Word rulesym Word1 ... Word2 or  Word
           rulesym []
@@ -405,13 +400,11 @@ def gRules(rules_list, rulesym="->", rhssep=None, rulesep='|'):
                         gr.append((lhs, l))
     return gr
 
-
 def smallAlphabet(k, sigma_base="a"):
     """Easy way to have small alphabets
 
       :param k: alphabet size (must be less than 52)
       :param sigma_base: initial symbol
-      
       :returns: alphabet
       :rtype: list"""
     Sigma = []
@@ -426,33 +419,68 @@ def smallAlphabet(k, sigma_base="a"):
             Sigma.append(chr(ord(sigma_base) + i))
     return Sigma
 
-
 reGrammar = {'g_regular_base': ["Tr -> Tr + Tc | Tc", "Tc -> Tc Ts |  Ts",
                                 "Ts -> Ts * | Ti | ( Tr ) "],
-             'g_regular_wredund': ["Tr -> Trs | f ", "Trs -> Trs + Tf | Tf + Tf",
+             'g_finite_base': ["Tr -> Tr + Tc | Tc", "Tc -> Tc Ts |  Ts",
+                                "Ts ->  Ti | ( Tr ) "],
+             'g_regular_wredund': ["Tr -> Trs | Tf ", "Trs -> Trs + Tf | Tf + Tf",
                                    "Tf -> Tc | Te | Ti",
                                    "Tc -> Tc Ts |  Ts Ts",
                                    "Ts -> Te | Ti | ( Trs ) ",
                                    "Te -> ( Trs ) * | ( Tc ) * | Ti *"],
-             'g_regular_uncollaps': ["Ts ->  Trs | Tcc  | Tee | Ti | %s | %s" % (common.Epsilon, common.EmptySet),
+             'g_regular_uncollaps': ["Ts ->  Trs | Tcc  | Tee | Ti | {0:s} | {1:s}".format(common.Epsilon, common.EmptySet),
                                      "Tcc -> Tcc Tr | Tr Tr",
                                      "Tr -> ( Trs ) | Tee | Ti ",
                                      "Tee -> ( Trs ) * | ( Tcc ) * | Ti *",
-                                     " Trs -> %s + Tx | Ty + Tz" % common.Epsilon,
+                                     "Trs -> {0:s} + Tx | Ty + Tz".format(common.Epsilon),
                                      "Tx -> Tt | Tt + Tx",
                                      "Tt -> Tcc | Ti ",
                                      "Ty -> Tz | Ty + Tz",
                                      "Tz -> Tcc | Tee | Ti "],
-             'g_rpn': ["Tx -> %s | Ti | +  Tx  Tx  | . Tx Tx  | * Tx" % common.Epsilon],
-             'g_sha': ["Ts  -> Ted | Tec | Tes | Ti |%s" % common.Epsilon,
+             'g_pt': [" Tz -> Tz + Tf | Tf",
+                      "Tf -> Tf {0:s}  Te | Te".format(common.Conj),
+                      "Te -> {0:s} ( Ts ) | Ts ".format(common.Not),
+                       "Ts -> Ts Tu | {0:s} Tu".format(common.SigmaS),
+                      "Tu ->  Ti {0:s}".format(common.SigmaS)],
+             'g_sha': ["Ts  -> Ted | Tec | Tes | Ti | {0:s}".format(common.Epsilon),
                        "Tec -> . Tec Tr | . Tr Tr ",
                        "Tr -> Ted | Tes | Ti",
                        "Tes -> * Ted | * Tec | * Ti",
-                       "Ted -> + %s Tx | + Ty Tz" % common.Epsilon,
+                       "Ted -> + {0:s} Tx | + Ty Tz".format(common.Epsilon),
                        "Tx -> Tt | + Tt Tx",
                        "Tt -> Tec | Ti",
                        "Ty -> Tz | + Ty Tz",
                        "Tz -> Tec | Tes | Ti"],
-             'g_rpn_pi': [ "Tp -> Ti | +  Tp  Tx | + Tnp Tp | . Tx Tp | . Tp %s" % common.Epsilon,
-                           "Tx -> %s | Ti | +  Tx  Tx  | . Tx Tx  | * Tx" % common.Epsilon,
-                           "Tnp -> %s | +  Tnp  Tnp | . Tnp Tnp | . Tp Tnp" % common.Epsilon]}
\ No newline at end of file
+             'g_rpn': ["Tx -> {0:s} | Ti | +  Tx  Tx  | . Tx Tx  | * Tx".format(common.Epsilon)],
+             'g_rpn_finite': ["Tx -> {0:s} | Ti | +  Tx  Tx  | . Tx Tx".format(common.Epsilon)],
+             'g_rpn_snf':["Tx ->  Tp | Tnp",
+                    "Tp -> {0:s} | . Tp Tp | + Tp Tnp | + Tnp Tp | + Tp Tp |  * Tnp ".format(common.Epsilon),
+                    "Tnp -> Ti | + Tnp Tnp | . Tp Tnp | . Tnp Tp | . Tnp Tnp"],
+             'g_rpn_snf_uf': ["Tx ->  Tp | Tnp",
+                           "Tp ->  . Tp Tp |   * Tnp ",
+                           "Tnp -> Ti |  . Tp Tnp | . Tnp Tp | . Tnp Tnp"],
+             'g_rpn_option':["Tp -> {0:s} | Tx ".format(common.Epsilon),
+                 "Tx ->  Ti | +  Tx  Tx  | . Tx Tx  | * Tx| {0:s} Tx".format(common.Option)],
+             'g_rpn_snf_option':["Tx -> {0:s} | Tp | Tnp".format(common.Epsilon),
+                    "Tp -> . Tp Tp | + Tp Tnp | + Tnp Tp | + Tp Tp |* Tnp| {0:s} Tnp".format(common.Option),
+                    "Tnp -> Ti | + Tnp Tnp | . Tp Tnp | . Tnp Tp | . Tnp Tnp"],
+             'g_rpn_pi': [ "Tp -> Ti | +  Tp  Tx | + Tnp Tp | . Tx Tp | . Tp {0:s}".format(common.Epsilon),
+                           "Tx -> {0:s} | Ti | +  Tx  Tx  | . Tx Tx  | * Tx".format(common.Epsilon),
+                           "Tnp -> {0:s} | +  Tnp  Tnp | . Tnp Tnp | . Tp Tnp".format(common.Epsilon)],
+             'grse_rpn': ["Tx -> {0:s} | Ti | +  Tx  Tx  | {1:s} Tx Tx | . Tx Tx  | * Tx".format(common.Epsilon, common.Shuffle)],
+             'grs_rpn': ["Tx ->  Ti | +  Tx  Tx  | {0:s} Tx Tx | . Tx Tx  | * Tx".format(common.Shuffle)],
+             'grf_rpn': ["Tx ->  Ti | +  Tx  Tx  | {0:s} Tx Tx | . Tx Tx ".format(common.Shuffle)],
+             'grce_rpn': ["Tx -> {0:s} | Ti | +  Tx  Tx  | {1:s} Tx Tx | . Tx Tx  | * Tx".format(common.Epsilon, common.Conj)],
+             'grc_rpn': ["Tx ->  Ti | +  Tx  Tx  | {0:s} Tx Tx | . Tx Tx  | * Tx".format(common.Conj)],
+             'grcf_rpn': ["Tx ->  Ti | +  Tx  Tx  | {0:s} Tx Tx | . Tx Tx ".format(common.Conj)],
+             'grcn_rpn': ["Tx ->  Ti | +  Tx  Tx  | {0:s} Tx Tx | . Tx Tx  | {1:s} Tx | * Tx".format(common.Conj, common.Not)],
+             'grche_rpn': ["Ts -> {0:s} Tx Tx".format(common.Conj), "Tx ->  {0:s} |  Ti | +  Tx  Tx  | . Tx Tx  | * Tx".format(common.Epsilon)],
+             'grcv_rpn': [ "Tn -> Ti | + Tn Tx | + Tv Tn | * Tn | . Tn Tx | . Tv Tn | {0:s} Tn Tn".format(common.Conj),
+                           "Tv -> {0:s} | + Tv Tv | * Tv | . Tv Tv | {1:s} Tv Tx | {1:s} Tx Tv".format(common.Epsilon, common.Conj),
+                          "Tx -> {0:s} | Ti | +  Tx  Tx  | {1:s} Tx Tx | . Tx Tx  | * Tx".format(common.Epsilon, common.Conj)],
+             'gpt_rpn':  [" Tz ->  + Tz  Tf | Tf",
+                      "Tf -> {0:s} Tf Te | Te".format(common.Conj),
+                      "Te -> {0:s} Ts | Ts ".format(common.Not),
+                       "Ts -> . Ts Tu | . {0:s} Tu".format(common.SigmaS),
+                      "Tu ->  . Ti {0:s}".format(common.SigmaS)]
+            }
