@@ -1,29 +1,29 @@
 # -*- coding: utf-8 -*-
 """**Several combined operations for DFAs**
 
-Deterministic and non-deterministic automata manipulation, conversion and evaluation.
+Combined operations
 
-*Authors:* Rogério Reis & Nelma Moreira
-
-*This is part of FAdo project*   http://fado.dcc.fc.up.pt
-
-*Version:* 0.9.5
+.. *Authors:* Rogério Reis & Nelma Moreira
 
 .. versionadded: 0.9.4
 
-*Copyright:* 1999-2012 Rogério Reis & Nelma Moreira {rvr,nam}@dcc.fc.up.pt
+.. *This is part of FAdo project*   http://fado.dcc.fc.up.pt.
 
+.. *Copyright:* 1999-2014 Rogério Reis & Nelma Moreira {rvr,nam}@dcc.fc.up.pt
 
-.. This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
-   License as published by the Free Software Foundation; either version 2 of the License,
-   or (at your option) any later version.
+.. This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as published
+   by the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-    details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-   You should have received a copy of the GNU General Public License along with this program; if not,
-   write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   675 Mass Ave, Cambridge, MA 02139, USA."""
 
 from fa import *
 from common import *
@@ -32,12 +32,9 @@ from common import *
 def starConcat(fa1, fa2, strict=False):
     """ Star of concatenation of two languages: (L1.L2)*
 
-    :param fa1: first automaton
-    :param fa2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type fa1: DFA
-    :type fa2: DFA
+    :param DFA f a1: first automaton
+    :param DFA fa2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
@@ -108,14 +105,14 @@ def starConcat(fa1, fa2, strict=False):
     for sym in c.Sigma:
         s1 = {d1.evalSymbol(d1.Initial, sym)}
         s2 = set([])
-        #Z1
+        # Z1
         if s1 & d1.Final != set([]):
             s2.add(d2.Initial)
             if d2.finalP(d2.Initial):
                 s1.add(d1.Initial)
         if d1.finalP(d1.Initial):
             s2.add(d2.evalSymbol(d2.Initial, sym))
-            #Z2
+            # Z2
             if s2 & d2.Final != set([]):
                 s1.add(d1.Initial)
                 s2.add(d2.Initial)
@@ -163,12 +160,9 @@ def starConcat(fa1, fa2, strict=False):
 def concatWStar(fa1, fa2, strict=False):
     """Concatenation combined with star: (L1.L2*)
 
-    :param fa1: first automaton
-    :param fa2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: bool
-    :type fa1: DFA
-    :type fa2: DFA
+    :param DFA fa1: first automaton
+    :param DFA fa2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
@@ -239,12 +233,9 @@ def concatWStar(fa1, fa2, strict=False):
 def starWConcat(fa1, fa2, strict=False):
     """Star combined with concatenation: (L1*.L2)
 
-    :param fa1: first automaton
-    :param fa2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type fa1: DFA
-    :type fa2: DFA
+    :param DFA fa1: first automaton
+    :param DFA fa2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
@@ -316,12 +307,9 @@ def starWConcat(fa1, fa2, strict=False):
 def starDisj(fa1, fa2, strict=False):
     """Star of Union of two DFAs: (L1 + L2)*
 
-    :param fa1: first automaton
-    :param fa2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type fa1: DFA
-    :type fa2: DFA
+    :param DFA fa1: first automaton
+    :param DFA fa2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
@@ -388,12 +376,9 @@ def starDisj(fa1, fa2, strict=False):
 def starInter0(fa1, fa2, strict=False):
     """Star of Intersection  of two DFAs: (L1 & L2)*
 
-    :param fa1: first automaton
-    :param fa2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type fa1: DFA
-    :type fa2: DFA
+    :param DFA fa1: first automaton
+    :param DFA fa2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
@@ -460,12 +445,9 @@ def starInter0(fa1, fa2, strict=False):
 def starInter(fa1, fa2, strict=False):
     """Star of Intersection  of two DFAs: (L1 & L2)*
 
-    :param fa1: first automaton
-    :param fa2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type fa1: DFA
-    :type fa2: DFA
+    :param DFA fa1: first automaton
+    :param DFA fa2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA """
     if strict and fa1.Sigma != fa2.Sigma:
         raise DFAdifferentSigma
@@ -537,12 +519,9 @@ def starInter(fa1, fa2, strict=False):
 def disjWStar(f1, f2, strict=True):
     """Union with star: (L1 + L2*)
 
-    :param f1: first automaton
-    :param f2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type f1: DFA
-    :type f2: DFA
+    :param DFA f1: first automaton
+    :param DFA f2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
@@ -556,12 +535,9 @@ def disjWStar(f1, f2, strict=True):
 def interWStar(f1, f2, strict=True):
     """Intersection with star: (L1 & L2*)
 
-    :param f1: first automaton
-    :param f2: second automaton
-    :param strict: should the alphabets be necessary equal?
-    :type strict: Boolean
-    :type f1: DFA
-    :type f2: DFA
+    :param DFA f1: first automaton
+    :param DFA f2: second automaton
+    :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
 
     .. seealso::
