@@ -23,11 +23,12 @@ FAdo IO.
    with this program; if not, write to the Free Software Foundation, Inc.,
    675 Mass Ave, Cambridge, MA 02139, USA."""
 
+import cStringIO
 from yappy_parser import Yappy, grules
-
-from common import Epsilon, DFAerror, TRError
+import common
 from fa import DFA, NFA, statePP
 from transducers import SFT, GFT, Transducer
+from fl import ADFA
 
 
 class ParserFAdo(Yappy):
@@ -39,8 +40,8 @@ class ParserFAdo(Yappy):
         tokenizer = [("\n+", lambda x: ("EOL", "EOL")),
                      ("#.*", ""),
                      ("\s+", ""),
-                     #("@epsilon", lambda x: ("ids", "@epsilon")),
-                     ("@epsilon", lambda x: ("ids", Epsilon)),
+                     # ("@epsilon", lambda x: ("ids", "@epsilon")),
+                     ("@epsilon", lambda x: ("ids", common.Epsilon)),
                      ("@NFA", lambda x: ("NFA", "NFA")),
                      ("@DFA", lambda x: ("DFA", "DFA")),
                      ("@TDFA", lambda x: ("TDFA", "TDFA")),
@@ -157,7 +158,7 @@ class ParserFAdo(Yappy):
         self.alphabet = set()
         self.alphabetOut = set()
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     @staticmethod
     def defaultSemRule(lst, context=None):
         """Defines the default semantic rule for Yappy
@@ -166,7 +167,7 @@ class ParserFAdo(Yappy):
         :returns: first argument semantics"""
         return lst[0]
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     @staticmethod
     def emptySemRule(lst, context=None):
         """Defines the empty semantic rule for Yappy
@@ -175,7 +176,7 @@ class ParserFAdo(Yappy):
         :returns: empty list"""
         return []
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def startDFASemRule(self, lst, context=None):
         """
 
@@ -199,7 +200,7 @@ class ParserFAdo(Yappy):
         self.theList.append(new)
         self.initLocal()
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def startTDFASemRule(self, lst, context=None):
         """
 
@@ -208,7 +209,7 @@ class ParserFAdo(Yappy):
         """
         pass
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def startNFASemRule(self, lst, context=None):
         """
 
@@ -231,7 +232,7 @@ class ParserFAdo(Yappy):
         self.theList.append(new)
         self.initLocal()
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def startTRANSSemRule(self, lst, context=None):
             """
 
@@ -241,7 +242,8 @@ class ParserFAdo(Yappy):
                 new = SFT()
             elif self.TRtype == "GFT":
                 new = GFT()
-            else: raise TRError
+            else:
+                raise common.TRError
             new.Sigma = self.alphabet
             new.Output = self.alphabetOut
             while self.states:
@@ -259,7 +261,7 @@ class ParserFAdo(Yappy):
             self.theList.append(new)
             self.initLocal()
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def finalSemRule(self, lst, context=None):
         """
 
@@ -268,7 +270,7 @@ class ParserFAdo(Yappy):
         self.finals.append(lst[0])
         self.states.add(lst[0])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def initialSemRule(self, lst, context=None):
         """
 
@@ -277,7 +279,7 @@ class ParserFAdo(Yappy):
         self.initials.append(lst[0])
         self.states.add(lst[0])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def firstTransitionSemRule(self, lst, context=None):
         """
 
@@ -288,7 +290,7 @@ class ParserFAdo(Yappy):
         self.states.add(lst[2])
         self.transitions.append((lst[0], lst[1], lst[2]))
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def firstTTransitionSemRule(self, lst, context=None):
         """
 
@@ -296,7 +298,7 @@ class ParserFAdo(Yappy):
         :param context:"""
         pass
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def firstTransitionTransSemRule(self, lst, context=None):
         """
 
@@ -305,11 +307,12 @@ class ParserFAdo(Yappy):
         self.initials.append(lst[0])
         self.states.add(lst[0])
         self.states.add(lst[3])
-        if self.TRtype is None and (lst[1] != Epsilon and len(lst[1]) > 1) or (lst[2] != Epsilon and len(lst[2]) > 1):
+        if self.TRtype is None and (lst[1] != common.Epsilon and len(lst[1]) > 1) or \
+                (lst[2] != common.Epsilon and len(lst[2]) > 1):
             self.TRtype = "GFT"
         self.transitions.append((lst[0], lst[1], lst[2], lst[3]))
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def firstDeclareState(self, lst, context=None):
         """
 
@@ -318,7 +321,7 @@ class ParserFAdo(Yappy):
         self.states.add(lst[0])
         self.initials.append(lst[0])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def addAlphabet(self, lst, context=None):
         """
 
@@ -326,7 +329,7 @@ class ParserFAdo(Yappy):
         :param context:"""
         self.alphabet.add(lst[0])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def addAlphabetOut(self, lst, context=None):
         """
 
@@ -334,7 +337,7 @@ class ParserFAdo(Yappy):
         :param context:"""
         self.alphabetOut.add(lst[0])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def declareState(self, lst, context=None):
         """
 
@@ -342,18 +345,18 @@ class ParserFAdo(Yappy):
         :param context:"""
         self.states.add(lst[0])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def firstTransitionESemRule(self, lst, context=None):
         """
 
         :param lst:
         :param context:"""
-        self.transitions.append((lst[0], Epsilon, lst[2]))
+        self.transitions.append((lst[0], common.Epsilon, lst[2]))
         self.initials.append(lst[0])
         self.states.add(lst[0])
         self.states.add(lst[2])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def transitionSemRule(self, lst, context=None):
         """
 
@@ -363,19 +366,19 @@ class ParserFAdo(Yappy):
         self.states.add(lst[0])
         self.states.add(lst[2])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def transitionTransSemRule(self, lst, context=None):
         """
 
         :param lst:
         :param context:"""
-        if self.TRtype is None and (lst[1] != Epsilon and len(lst[1]) > 1) or (lst[2] != Epsilon and len(lst[2]) > 1):
+        if self.TRtype is None and (lst[1] != common.Epsilon and len(lst[1]) > 1) or (lst[2] != common.Epsilon and len(lst[2]) > 1):
             self.TRtype = "GFT"
         self.transitions.append((lst[0], lst[1], lst[2], lst[3]))
         self.states.add(lst[0])
         self.states.add(lst[3])
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def transitionTSemRule(self, lst, context=None):
         """
 
@@ -383,13 +386,13 @@ class ParserFAdo(Yappy):
         :param context:"""
         pass
 
-    #noinspection PyUnusedLocal
+    # noinspection PyUnusedLocal
     def transitionESemRule(self, lst, context=None):
         """
 
         :param lst:
         :param context:"""
-        self.transitions.append((lst[0], Epsilon, lst[2]))
+        self.transitions.append((lst[0], common.Epsilon, lst[2]))
         self.states.add(lst[0])
         self.states.add(lst[2])
 
@@ -412,8 +415,7 @@ def readOneFromFile(fileName):
 def readFromFile(FileName):
     """Reads list of finite automata definition from a file.
 
-    :param FileName: file name
-    :type FileName: str
+    :param str FileName: file name
     :rtype: list
 
     The format of these files must be the as simple as possible:
@@ -476,6 +478,164 @@ def readOneFromString(s):
     return parser.result()[0]
 
 
+def alphabetPP(sigma):
+        ssig = "[ \"{0:>s}\"".format(str(sigma.pop()))
+        for sym in sigma:
+            ssig += ", \"{0:>s}\"".format(str(sym))
+        ssig += " ]"
+        return ssig
+
+
+def toJson(aut):
+    """
+    Json for a FA
+    :param fa:
+    :return:
+    """
+    io = cStringIO.StringIO()
+    if isinstance(aut, DFA):
+        jtype = "DFA"
+    elif isinstance(aut, NFA):
+        jtype = "NFA"
+    elif isinstance(aut, Transducer):
+        jtype = "Transducer"
+    elif isinstance(aut, ADFA):
+        jtype = "ADFA"
+    io.write("{ \"automaton\": {\n\t\"title\": \"\", \n\t\"version\": \"\",\n")
+    io.write("\t\"type\": \"{0:>s}\",\n".format(jtype))
+    io.write("\t\"states\": [\n")
+    sn = 0
+    for s in xrange(len(aut.States)):
+        if sn == 0:
+            io.write("{ \n")
+        else:
+            io.write(",\n{ \n")
+        io.write("\t\t\"name\": \"{0:>s}\",\n".format(str(s)))
+        io.write("\t\t\"label\": \"{0:>s}\",\n".format(statePP(aut.States[s])))
+        io.write("\t\t\"output\": \"\",\n")
+        if aut.initialP(s):
+            io.write("\t\t\"initial\": true,\n")
+        else:
+            io.write("\t\t\"initial\":false,\n")
+        if aut.finalP(s):
+            io.write("\t\t\"final\": true\n")
+        else:
+            io.write("\t\t\"final\": false\n")
+        io.write("}")
+        sn += 1
+    io.write("], \n")
+    io.write("\t\"trans\": [\n")
+    trn = 0
+    for s in xrange(len(aut.States)):
+        if s in aut.delta:
+            for a in aut.delta[s].keys():
+                if isinstance(aut.delta[s][a], set):
+                    for s1 in aut.delta[s][a]:
+                        if trn == 0:
+                            io.write("{ \n")
+                        else:
+                            io.write(",\n{ \n")
+                        io.write("\t\t\"name\": \"{0:>s}\,\n".format(str(trn)))
+                        io.write("\t\t\"orig_name\": \"{0:>s}\,\n".format(statePP(aut.States[s])))
+                        io.write("\t\t\"dest_name\": \"{0:>s}\",\n".format(statePP(aut.States[s1])))
+                        io.write("\t\t\"label\": \"{0:>s}\",\n".format(str(a))),
+                        io.write("\t\t\"weight\": \"\"\n")
+                        io.write("}")
+                        trn += 1
+                else:
+                    if trn == 0:
+                        io.write("{ \n")
+                    else:
+                        io.write(",\n{ \n")
+                    # io.write(", \n{ \n")
+                    io.write("\t\t\"name\": \"{0:>s}\",\n".format(str(trn)))
+                    io.write("\t\t\"orig_name\": \"{0:>s}\",\n".format(statePP(aut.States[s])))
+                    io.write("\t\t\"dest_name\": \"{0:>s}\",\n".format(statePP(aut.States[aut.delta[s][a]])))
+                    io.write("\t\t\"label\": \"{0:>s}\",\n".format(str(a))),
+                    io.write("\t\t\"weight\": \"\"\n")
+                    io.write("}")
+                    trn += 1
+    io.write("],\n")
+    io.write("\t\"alphabet\": {0:>s} \n".format(alphabetPP(aut.Sigma)))
+    io.write(" } ")
+    return io.getvalue()
+
+
+def saveToJson(FileName, aut, mode="w"):
+    """ Saves a finite automata definition to a file using the JSON format
+    """
+    try:
+        f = open(FileName, mode)
+    except IOError:
+        raise common.DFAerror()
+    f.write(toJson(aut))
+    f.close()
+
+
+def saveToString(fa):
+    """ Saves a finite automaton definition to a string
+    :param fa: automaton
+    :return: the string containing the automaton definition
+    :rtype: str
+
+    ..versionadded:: 1.2.1"""
+
+    def _save_SFTransducer(tr, io):
+        io.write("@Transducer ")
+        for s in tr.Final:
+            io.write("{0:>s} ".format(statePP(tr.States[s])))
+        io.write("* ")
+        for s in tr.Initial:
+            io.write("{0:>s} ".format(statePP(fa.States[s])))
+        io.write("\n")
+        for sin in tr.delta:
+            for syin in tr.delta[sin]:
+                for (syout, sout) in tr.delta[sin][syin]:
+                    io.write("{0:>s} {1:>s} {2:>s} {3:>s}\n".format(statePP(tr.States[sin]), str(syin), str(syout),
+                                                                    statePP(tr.States[sout])))
+        io.write("\n")
+
+    def _saveFA(aut, io):
+        if isinstance(aut, DFA):
+            io.write("@DFA ")
+            NFAp = False
+        elif isinstance(aut, NFA):
+            io.write("@NFA ")
+            NFAp = True
+        else:
+            raise common.DFAerror()
+        if not NFAp and aut.Initial != 0:
+            foo = {0: aut.Initial, aut.Initial: 0}
+            aut.reorder(foo)
+        for sf in aut.Final:
+            io.write("{0:>s} ".format(statePP(aut.States[sf])))
+        if NFAp:
+            io.write(" * ")
+            for sf in aut.Initial:
+                io.write("{0:>s} ".format(statePP(aut.States[sf])))
+        io.write("\n")
+        for s in xrange(len(aut.States)):
+            if s in aut.delta:
+                for a in aut.delta[s].keys():
+                    if isinstance(aut.delta[s][a], set):
+                        for s1 in aut.delta[s][a]:
+                            io.write("{0:>s} {1:>s} {2:>}\n".format(statePP(aut.States[s]),
+                                                                    str(a), statePP(aut.States[s1])))
+                    else:
+                        io.write("{0:>s} {1:>s} {2:>s}\n".format(statePP(aut.States[s]), str(a),
+                                                                 statePP(aut.States[aut.delta[s][a]])))
+            else:
+                io.write("{0:>s} \n".format(statePP(aut.States[s])))
+
+    out = cStringIO.StringIO()
+    if isinstance(fa, Transducer):
+        _save_SFTransducer(fa, out)
+        return out.getvalue()
+    else:
+        _saveFA(fa, out)
+        return out.getvalue()
+
+
 def saveToFile(FileName, fa, mode="a"):
     """ Saves a list finite automata definition to a file using the input format
 
@@ -483,84 +643,21 @@ def saveToFile(FileName, fa, mode="a"):
     .. versionchanged:: 0.9.6
     .. versionchanged:: 0.9.7 New format with quotes and alphabet
 
-    :param FileName: file name
-    :type FileName: str
+    :param str FileName: file name
     :param fa: the FA
     :type fa: list of FA
-    :param mode: writing mode
-    :type mode: str """
-
-    #TODO: write the complete information into file according with the new format
-    def _save_SFTransducer(tr):
-        """Writes transducer to a file
-
-        :param tr: the transducer
-        :type tr: Transducer"""
-        f.write("@Transducer ")
-        for s in tr.Final:
-            f.write("{0:>s} ".format(statePP(tr.States[s])))
-        f.write("* ")
-        for s in tr.Initial:
-            f.write("{0:>s} ".format(statePP(fa.States[s])))
-        f.write("\n")
-        for sin in tr.delta:
-            for syin in tr.delta[sin]:
-                for (syout, sout) in tr.delta[sin][syin]:
-                    f.write("{0:>s} {1:>s} {2:>s} {3:>s}\n".format(statePP(tr.States[sin]), str(syin), str(syout),
-                                                                   statePP(tr.States[sout])))
-        f.write("\n")
-
-    def _saveFA(aut):
-        """Writes one fa to a file
-
-        :param aut: the automaton
-        :type aut: FA or list"""
-        if isinstance(aut, DFA):
-            f.write("@DFA ")
-            NFAp = False
-        elif isinstance(aut, NFA):
-            f.write("@NFA ")
-            NFAp = True
-        else:
-            raise DFAerror()
-        if not NFAp and aut.Initial != 0:
-            foo = {0: aut.Initial, aut.Initial: 0}
-            aut.reorder(foo)
-        for sf in aut.Final:
-            f.write("{0:>s} ".format(statePP(aut.States[sf])))
-        if NFAp:
-            f.write(" * ")
-            for sf in aut.Initial:
-                f.write("{0:>s} ".format(statePP(aut.States[sf])))
-        f.write("\n")
-        for s in xrange(len(aut.States)):
-            if s in aut.delta:
-                for a in aut.delta[s].keys():
-                    if isinstance(aut.delta[s][a], set):
-                        for s1 in aut.delta[s][a]:
-                            f.write(
-                                "{0:>s} {1:>s} {2:>}\n".format(statePP(aut.States[s]), str(a), statePP(aut.States[s1])))
-                    else:
-                        f.write("{0:>s} {1:>s} {2:>s}\n".format(statePP(aut.States[s]), str(a),
-                                                                statePP(aut.States[aut.delta[s][a]])))
-            else:
-                f.write("{0:>s} \n".format(statePP(aut.States[s])))
+    :param str mode: writing mode"""
 
+    # TODO: write the complete information into file according with the new format
     try:
         f = open(FileName, mode)
     except IOError:
-        raise DFAerror()
+        raise common.DFAerror()
     if type(fa) == list:
         for d in fa:
-            if isinstance(d, Transducer):
-                _save_SFTransducer(d)
-            else:
-                _saveFA(d)
+            f.write(saveToString(d))
     else:
-        if isinstance(fa, Transducer):
-            _save_SFTransducer(fa)
-        else:
-            _saveFA(fa)
+        f.write(saveToString(fa))
     f.close()
 
 
@@ -570,16 +667,14 @@ def _exportToTeX(FileName, fa):
 
     .. versionchanged:: 0.9.4
 
-    :param FileName: file name
-    :type FileName: str
-    :param fa: the FA
-    :type fa: FA
+    :param str FileName: file name
+    :param FA fa: the FA
     :raises DFAerror: if a file error occurs"""
     try:
         f = open(FileName, "w")
     except IOError:
-        raise DFAerror()
-        #initial is the first one
+        raise common.DFAerror()
+        # initial is the first one
     if fa.Initial:
         foo = {0: fa.Initial, fa.Initial: 0}
         fa.reorder(foo)
@@ -609,3 +704,11 @@ def _exportToTeX(FileName, fa):
             f.write("\\\\\n")
     f.write("\end{array}$$")
     f.close()
+
+
+def show(obj):
+    """ General, context sensitive, display method
+    :param obj: the object to show
+
+    .. versionaded:: 1.2.1 """
+    pass
