@@ -7,7 +7,7 @@ Finite languages manipulation
 
 .. *This is part of FAdo project*   <http://fado.dcc.fc.up.pt
 
-.. *Version:* 0.9.5
+.. *Version:* 1.3.3
 
 .. *Copyright*: 1999-2014 Rogério Reis & Nelma Moreira {rvr,nam}@dcc.fc.up.pt
 
@@ -26,7 +26,7 @@ Finite languages manipulation
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
 
 import fa
-from copy import copy, deepcopy
+from copy import copy
 from common import *
 import random
 
@@ -36,18 +36,28 @@ class FL(object):
 
     :var Words: the elements of the language
     :var Sigma: the alphabet"""
-    def __init__(self, wordsList=None, Sigma=set([])):
-        if not wordsList:
-            wordsList = []
-        self.Words = set(wordsList)
-        self.Sigma = Sigma
-        for w in self.Words:
-            if w != Epsilon:
-                for l in w:
-                    self.Sigma.add(l)
+    def __init__(self, wordsList=None, Sigma=None):
+        self.Words = set([])
+        if Sigma is None:
+            self.Sigma = set([])
+        else:
+            self.Sigma = Sigma
+        if wordsList is not None:
+            for w in wordsList:
+                if type(w) is Word:
+                    self.addWord(w)
+                else:
+                    self.addWord(Word(w))
 
     def __str__(self):
-        return "(%s,%s)" % (list(self.Words), list(self.Sigma))
+        l = "({"
+        for i in self.Words:
+            l += str(i) + ","
+        l = l[:-1] + "}, {"
+        for i in self.Sigma:
+            l += str(i)+","
+        l = l[:-1] + "})"
+        return l
 
     def __repr__(self):
         return "FL%s" % self.__str__()
@@ -55,8 +65,11 @@ class FL(object):
     def __len__(self):
         return len(self.Words)
 
-    def reunion(self, other):
-        """Reunion of FL:   a | b
+    def __contains__(self, item):
+        return item in self.Words
+
+    def union(self, other):
+        """union of FL:   a | b
 
         :param FL other: right hand operand
         :rtype: FL
@@ -78,6 +91,9 @@ class FL(object):
         :raises FAdoGeneralError: if both arguments are not FL"""
         return self.__and__(other)
 
+    def __iter__(self):
+        return iter(self.Words)
+
     def __and__(self, other):
         if type(other) != type(self):
             raise FAdoGeneralError("Incompatible objects")
@@ -120,11 +136,24 @@ class FL(object):
         """Adds a list of words to a FL
 
         :param list wList: words to add"""
-        self.Words = self.Words | set(wList)
+        self.Words |= set(wList)
         for w in wList:
+            if w.epsilonP():
+                continue
             for c in w:
                 self.Sigma.add(c)
 
+    def addWord(self, word):
+        """Adds a word to a FL
+        :type word: Word
+        :rtype: FL"""
+        if word in self:
+            return
+        self.Words.add(word)
+        if not word.epsilonP():
+            for c in word:
+                self.Sigma.add(c)
+
     def suffixClosedP(self):
         """Tests if a language is suffix closed
 
@@ -161,6 +190,30 @@ class FL(object):
                 b.addWords([w])
         return a, b
 
+    def MADFA(self):
+        """Generates the minimal acyclical DFA using specialized algorithm
+
+        .. versionadded:: 1.3.3
+
+        .. seealso:: Incremental Construction of Minimal Acyclic Finite-State Automata, J.Daciuk, S.Mihov, B.Watson and R.E.Watson
+
+        :rtype: ADFA"""
+        aut = ADFA()
+        i = aut.addState()
+        aut.setInitial(i)
+        aut.setSigma(self.Sigma)
+        register = set()
+        foo = sorted(list(self.Words))
+        for w in foo:  # sorted(list(self.Words)):
+            (cPrefix, lState) = aut._common_prefix(w)
+            cSuffix = w[len(cPrefix):]
+            if aut.delta.get(lState, {}):
+                aut._replace_or_register(lState, register)
+            aut.addSuffix(lState, cSuffix)
+        aut._replace_or_register(i, register)
+        aut.Minimal = True
+        return aut
+
     def trieFA(self):
         """Generates the trie automaton that recognises this language
 
@@ -171,7 +224,7 @@ class FL(object):
         i = new.addState()
         new.setInitial(i)
         for w in self.Words:
-            if w == Epsilon:
+            if w.epsilonP():
                 new.addFinal(i)
             else:
                 s = i
@@ -185,12 +238,29 @@ class FL(object):
                 new.addFinal(s)
         return new
 
+    def toDFA(self):
+        """ Generates a DFA recognizing the language
+
+        :rtype: ADFA
+
+        .. versionadded:: 1.2"""
+        return self.trieFA()
+
+    def toNFA(self):
+        """Generates a NFA recognizing the language
+
+        :rtype: ANFA
+
+        .. versionadded:: 1.2"""
+        return self.toDFA().toANFA()
+
     # noinspection PyUnboundLocalVariable
     def multiLineAutomaton(self):
         """Generates the trivial linear ANFA equivalent to this language
     
         :rtype: ANFA"""
         new = ANFA()
+        s1 = None
         new.setSigma(copy(self.Sigma))
         for w in self.Words:
             s = new.addState()
@@ -203,13 +273,13 @@ class FL(object):
         return new
 
 
-class DCFA(fa.DFA):
+class DFCA(fa.DFA):
     """Deterministic Cover Automata class
 
-    .. inheritance-diagram:: DCFA"""
+    .. inheritance-diagram:: DFCA"""
 
     def __init__(self):
-        super(DCFA, self).__init__()
+        super(DFCA, self).__init__()
         self.length = None
 
     @property
@@ -240,6 +310,20 @@ class AFA(object):
        This is just a container for some common methods. **Not to be used directly!!**"""
     def __init__(self):
         self.Dead = None
+        self.delta = dict()
+        self.Initial = None
+        self.States = []
+        self.Final = set()
+
+    @abstractmethod
+    def addState(self):
+        """
+        :rtype: int"""
+        pass
+
+    @abstractmethod
+    def finalP(self):
+        pass
 
     def setDeadState(self, sti):
         """Identifies the dead state
@@ -258,7 +342,7 @@ class AFA(object):
         try:
             _ = self.Dead
         except AttributeError:
-            x = self.addState()
+            x = self.addState(DeadName)
             self.setDeadState(x)
 
     def ordered(self):
@@ -331,7 +415,7 @@ class AFA(object):
     def evalRank(self):
         """Evaluates the rank map of a automaton
 
-        :return: pair sets of states by rank map, reverse delta acessability map
+        :return: pair of sets of states by rank map, reverse delta accessability map
         :rtype: tuple"""
         (deltaC, rdelta) = self._getRdelta()
         rank, deltai = {}, {}
@@ -362,6 +446,7 @@ class AFA(object):
         :return: set of leaves
         :rtype: set"""
 
+        # noinspection PyUnresolvedReferences
         def _last(s1):
             queue, done = {s1}, set()
             while queue:
@@ -385,7 +470,15 @@ class AFA(object):
 class ADFA(fa.DFA, AFA):
     """Acyclic Deterministic Finite Automata class
 
-    .. inheritance-diagram:: ADFA"""
+    .. inheritance-diagram:: ADFA
+
+    .. versionchanged:: 1.3.3
+    """
+    def __init__(self):
+        fa.DFA.__init__(self)
+        AFA.__init__(self)
+        self.Minimal = False
+
     def __repr__(self):
         return 'ADFA({0:s})'.format(self.__str__())
 
@@ -393,21 +486,29 @@ class ADFA(fa.DFA, AFA):
         """Make the ADFA complete
 
         :param int dead: a state to be identified as dead state if one was not identified yet
+        :rtype: ADFA
 
         .. attention::
-           The object is modified in place"""
+           The object is modified in place
+
+        .. versionchanged:: 1.3.3"""
         if dead is not None:
             self.Dead = dead
         else:
             try:
-                _ = self.Dead
+                if self.Dead is None:
+                    raise AttributeError
+                else:
+                    _ = self.Dead
             except AttributeError:
-                foo = self.addState()
+                foo = self.addState(DeadName)
                 self.Dead = foo
         for st in range(len(self.States)):
             for k in self.Sigma:
                 if k not in self.delta.get(st, {}).keys():
                     self.addTransition(st, k, self.Dead)
+        self.Minimal = False
+        return self
 
     def dup(self):
         """Duplicate the basic structure into a new ADFA. Basically a copy.deep.
@@ -415,18 +516,33 @@ class ADFA(fa.DFA, AFA):
         :rtype: ADFA"""
         return deepcopy(self)
 
+    def __invert__(self):
+        """ Complement of a ADFA is a DFA
+
+        :return:DFA
+        """
+        aut = self.forceToDFA()
+        return ~aut
+
     def minimalP(self, method=None):
         """Tests if the DFA is minimal
 
         :param method: minimization algorithm (here void)
-        :rtype: bool"""
+        :rtype: bool
+
+        .. versionchanged:: 1.3.3"""
+        if self.Minimal:
+            return True
         foo = self.minimal()
         if self.completeP():
             foo.complete()
-        return len(foo) == len(self)
+        answ = len(foo) == len(self)
+        if answ:
+            self.Minimal = True
+        return answ
 
-    def forceToDCFA(self):
-        """ Conversion to DCFA
+    def forceToDFA(self):
+        """ Conversion to DFA
 
         :rtype: DFA"""
         new = fa.DFA()
@@ -439,11 +555,42 @@ class ADFA(fa.DFA, AFA):
                 new.addTransition(s, c, self.delta[s][c])
         return new
 
+    def forceToDFCA(self):
+        """ Conversion to DFCA
+
+        :rtype: DFA"""
+        new = fa.DFA()
+        new.States = deepcopy(self.States)
+        new.Sigma = deepcopy(self.Sigma)
+        new.Initial = self.Initial
+        new.Final = copy(self.Final)
+        for s in self.delta:
+            for c in self.delta[s]:
+                new.addTransition(s, c, self.delta[s][c])
+        return new
+
+    def wordGenerator(self):
+        """Creates a random word generator
+
+        :return: the random word generator
+        :rtype: RndWGen
+
+        .. versionadded:: 1.2"""
+        return RndWGen(self)
+
+    def possibleToReverse(self):
+        """Tests if language is reversible
+
+        .. versionadded:: 1.3.3"""
+        return True
+
     def minimal(self):
         """Finds the minimal equivalent ADFA
 
         .. seealso:: [TCS 92 pp 181-189] Minimisation of acyclic deterministic automata in linear time, Dominique Revuz
 
+        .. versionchanged:: 1.3.3
+
         :returns: the minimal equivalent ADFA
         :rtype: ADFA"""
 
@@ -470,7 +617,11 @@ class ADFA(fa.DFA, AFA):
         new = deepcopy(self)
         new.trim()
         new.complete()
-        (rank, rdelta) = new.evalRank()
+        if new.Dead is None:
+            deadName = None
+        else:
+            deadName = new.States[new.Dead]
+        rank, rdelta = new.evalRank()
         toBeDeleted = []
         maxr = len(rank) - 2
         for r in xrange(maxr + 1):
@@ -485,8 +636,231 @@ class ADFA(fa.DFA, AFA):
                     (d0, s0) = (d1, s1)
                 j += 1
         new.deleteStates(toBeDeleted)
+        if deadName is not None:
+            new.Dead = new.stateIndex(deadName)
+        new.Minimal = True
         return new
 
+    def minReversible(self):
+        """Returns the minimal reversible equivalent automaton
+
+        :rtype: ADFA"""
+        new = self.dup()
+        rank, rdelta = new.evalRank()
+
+
+
+    def statePairEquiv(self, s1, s2):
+        """Tests if two states of a ADFA are equivalent
+
+        :param int s1: state1
+        :param int s2: state2
+        :rtype: bool
+
+        .. versionadded:: 1.3.3"""
+        if not self.same_nullability(s1, s2):
+            return False
+        else:
+            return self.delta.get(s1, {}) == self.delta.get(s2, {})
+
+    def addSuffix(self, st, w):
+        """Adds a suffix starting in st
+
+        :param int st: state
+        :param Word w: suffix
+
+        .. versionadded:: 1.3.3
+
+        .. attention:: in place transformation"""
+        s1 = st
+        for c in w:
+            s2 = self.addState()
+            self.addTransition(s1, c, s2)
+            s1 = s2
+        self.addFinal(s1)
+
+    def _last_child(self, s):
+        """to be used by xxx of FL.MADFA
+
+        :param int s: state index
+        :returns: pair state index / symbol
+
+        .. versionadded:: 1.3.3"""
+        for c in sorted(list(self.Sigma)).__reversed__():
+            if c in self.delta.get(s, {}):
+                return self.delta[s][c], c
+        raise FAdoGeneralError("Something unexpected in _last_child({:d})".format(s))
+
+    def _replace_or_register(self, s, r):
+        """to be used by xxx of FL.MADFA
+
+        :param int s: state
+        :param Set r: register (inherited from context)
+
+        .. versionadded:: 1.3.3"""
+        (child, c) = self._last_child(s)
+        if self.delta.get(child, {}):
+            self._replace_or_register(child, r)
+        for q in r:
+            if self.statePairEquiv(q, child):
+                self.delta[s][c] = q
+                self.deleteState(child)
+                return
+        r.add(child)
+
+    def _common_prefix(self, wrd):
+        """The longest prefix of w that can be read in the ADFA and the correspondent state
+
+        :param Word wrd: word"""
+        pref = Word()
+        q = self.Initial
+        for s in wrd:
+            if s in self.delta.get(q, {}):
+                pref.append(s)
+                q = self.delta[q][s]
+            else:
+                break
+        return pref, q
+
+    def _addWordToMinimal(self, w):
+        """Incremental minimization algorithm
+
+        :param Word w: word
+
+        .. attention:: in place transformation
+
+        .. versionadded:: 1.3.3
+
+        .. seealso:: Incremental Construction of Minimal Acyclic Finite-State Automata, J.Daciuk, S.Mihov,
+                     B.Watson and R.E.Watson"""
+
+        def _transverseNonConfluence(wrd):
+            inCount = dict()
+            for s in range(len(self.States)):
+                for c in self.delta.get(s, {}):
+                    for s1 in self.delta[s][c]:
+                        inCount[s1] = inCount.get(s1, 0) + 1
+            q = self.Initial
+            visited = [q]
+            for i, sym in enumerate(wrd):
+                if sym not in self.delta.get(s, {}) or inCount[self.delta[q][sym]] > 1:
+                    return q, i
+                q = self.delta[q][sym]
+                visited.append(q)
+
+        def _cloneConfluence(st, wrd, ind):
+            q = st
+            for i, sym in enumerate(wrd[ind:]):
+                if sym not in self.delta.get(q, {}):
+                    return q, i + ind
+                qn = self.delta[q][sym]
+                sn = self.addState()
+                for c1 in self.delta.get(qn, {}):
+                    self.delta.setdefault(sn, {})[c1] = self.delta[qn][c1]
+                    if self.finalP(qn):
+                        self.addFinal(sn)
+                self.addTransition(q, sym, sn)
+                q = sn
+                visited.append(q)
+
+        def _replOrReg(st, wrd):
+            if len(w) != 0:
+                self.addTransition(q,wrd[0],_replOrReg(self.delta[st][wrd[0]], wrd[1:]))
+            else:
+                for c in register:
+                    if self.statePairEquiv(c, q):
+                        self.deleteState(q)
+                        return c
+                register.add(q)
+
+        def _addSuffix(st, wrd):
+            s = st
+            for c in wrd:
+                sn = self.addState()
+                self.addTransition(s, c, sn)
+                s = sn
+            self.addFinal(s)
+
+        register = set()
+        visited = []
+        q, i = _transverseNonConfluence(w)
+        f = q
+        register.remove(q)
+        j = i
+        q, i = _cloneConfluence(q, w, i)
+        _addSuffix(q, w[i:])
+        if j < len(w):
+            self.delta[f][w[j]] = _replOrReg(self.delta[f][w[j]], w[j+1:])
+
+
+
+    def dissMin(self, witnesses=None):
+        """Evaluates the minimal dissimilarity language
+        :param dict witnesses: optional witness dictionay
+        :rtype: FL
+
+        .. versionadded:: 1.2.1"""
+        new = self.minimal()
+        sz = len(new.States)
+        todo = [(i, j) for i in range(sz) for j in range(i)]
+        mD = FL(Sigma=new.Sigma)
+        lvl = new.level()
+        rnk = new.directRank()
+        l = max([rnk[x] for x in rnk])
+        Li = []
+        for (i, j) in todo:
+            if self.finalP(i) ^ self.finalP(j):
+                if witnesses is not None:
+                    witnesses[(i, j)] = Word(Epsilon)
+                Li.append((i, j))
+                mD.addWord(Word(Epsilon))
+        delFromList(todo, Li)
+        words = self.words()
+        for w in words:
+            if len(w) >= l or not todo:
+                break
+            Li = []
+            for (i, j) in todo:
+                if (lvl[i] + len(w) > l) or (lvl[j] + len(w) > l):
+                    Li.append((i, j))
+                elif self.evalWordP(w, i) ^ self.evalWordP(w, j):
+                    mD.addWord(w)
+                    if witnesses is not None:
+                        witnesses[(i, j)] = w
+                    Li.append((i, j))
+            delFromList(todo, Li)
+        return mD
+
+    def diss(self):
+        """ Evaluates the dissimilarity language
+
+        :rtype: FL
+
+        .. versionadded:: 1.2.1"""
+        new = self.minimal()
+        n = len(new.States)
+        mD = FL(Sigma=new.Sigma)
+        lvl = new.level()
+        rnk = new.directRank()
+        l = max([rnk[x] for x in rnk])
+        if len(new.Final) != n:
+            mD.addWord(Word(Epsilon))
+        words = self.words()
+        for w in words:
+            lw = len(w)
+            if lw >= l:
+                break
+            skip = False
+            for i in range(n):
+                if skip:
+                    break
+                for j in range(i):
+                    if (lvl[i] + lw <= l) and (lvl[j] + lw <= l) and (self.evalWordP(w, i) ^ self.evalWordP(w, j)):
+                        mD.addWord(w)
+                        skip = True
+                        break
+        return mD
+
     def level(self):
         """Computes the level  for each state
 
@@ -545,20 +919,20 @@ class ADFA(fa.DFA, AFA):
                             g = gp[(j1, i1)]
                         if g + 1 <= _range(self.stateIndex(i), self.stateIndex(j)):
                             gp[(self.stateIndex(i), self.stateIndex(j))] = min(gp[(self.stateIndex(i),
-                                                                                 self.stateIndex(j))], g + 1)
+                                                                                   self.stateIndex(j))], g + 1)
         return gp
 
     def minDFCA(self):
         """Generates a minimal deterministic cover automata from a DFA
 
-        :rtype: DCFA
+        :rtype: DFCA
 
         .. versionadded:: 0.9.8
 
         .. seealso::
             Cezar Campeanu, Andrei Päun, and Sheng Yu, An efficient algorithm for constructing minimal cover
             automata for finite languages, IJFCS"""
-        new = self.dup()
+        new = self.dup().minimal()
         if not self.completeP():
             new.complete()
         rank = new.directRank()
@@ -566,13 +940,15 @@ class ADFA(fa.DFA, AFA):
                      for v in set(rank.values()))
         l = rank[new.Initial]
         lvl = new.level()
-        lnames = []
         foo = [x for x in irank]
         foo.sort(reverse=True)
+        lnames = [None for _ in new.States]
+        newname = 0
         for i in foo:
             for j in irank[i]:
-                lnames.append(j)
-        new.renameStates(lnames)
+                lnames[j] = newname
+                newname += 1
+        new.States = lnames
         g = new._gap(l, lvl)
         P = [False for _ in new.States]
         toMerge = []
@@ -580,13 +956,13 @@ class ADFA(fa.DFA, AFA):
             if not P[i]:
                 for j in range(i + 1, len(new.States)):
                     if not P[j] and g[(new.stateIndex(i), new.stateIndex(j))] == l:
-                        toMerge.append((new.stateIndex(i), new.stateIndex(j)))
+                        toMerge.append((i, j))
                         P[j] = True
         for (a, b) in toMerge:
-            new.mergeStates(b, a)
+            new.mergeStates(new.stateIndex(b), new.stateIndex(a))
         new.trim()
-        new = new.forceToDCFA()
-        new.Length = l
+        new = new.forceToDFCA()
+        new.length = l
         return new
 
     def trim(self):
@@ -616,7 +992,69 @@ class ADFA(fa.DFA, AFA):
             new.addFinal(s)
         return new
 
+    def toNFA(self):
+        """Converts the ADFA in a equivalent NFA
 
+        :rtype: ANFA
+
+        .. versionadded:: 1.2"""
+        return self.toANFA()
+
+
+class RndWGen(object):
+    """Word random generator class
+
+    .. versionadded:: 1.2"""
+    def __init__(self, aut):
+        """
+        :param aut: automata recognizing the language
+        :type aut: ADFA """
+        self.Sigma = list(aut.Sigma)
+        self.table = dict()
+        self.aut = aut.minimal()
+        rank, _ = self.aut.evalRank()
+        self.aut._compute_delta_inv()
+        deltai = self.aut.delta_inv
+        mrank = max(rank)
+        for i in range(0, mrank + 1):
+            for s in rank[i]:
+                self.table.setdefault(s, {})
+                if self.aut.finalP(s):
+                    final = 1
+                else:
+                    final = 0
+                self.table[s][None] = sum([self.table[s].get(c, 0) for c in self.Sigma])
+                for c in self.Sigma:
+                    rs = deltai[s].get(c, [])
+                    for r in rs:
+                        self.table.setdefault(r, {})
+                        self.table[r][c] = self.table[s][None] + final
+
+    @staticmethod
+    def _rndChoose(l):
+        sm = sum(l)
+        r = random.randint(1, sm)
+        for i, j in enumerate(l):
+            if r <= j:
+                return i
+            else:
+                r -= j
+
+    def next(self):
+        """Next word
+
+        :return: a new random word"""
+        word = Word()
+        s = self.aut.Initial
+        while True:
+            if self.aut.finalP(s) and random.randint(1, self.table[s][None] + 1) == 1:
+                return word
+            i = self._rndChoose([self.table[s].get(c, 0) for c in self.Sigma])
+            word.append(self.Sigma[i])
+            s = self.aut.delta[s][self.Sigma[i]]
+
+
+# noinspection PyUnresolvedReferences
 class ANFA(fa.NFA, AFA):
     """Acyclic Nondeterministic Finite Automata class
 
@@ -631,7 +1069,7 @@ class ANFA(fa.NFA, AFA):
            stf must be a 'last' final state, i.e., must have no out transitions to anywhere but to a possible dead
            state
 
-        .. @attention:: the object is modified in place"""
+        .. attention:: the object is modified in place"""
         (rdelta, _) = self._getRdelta()
         for s in rdelta[st]:
             l = []
@@ -683,7 +1121,7 @@ class ANFA(fa.NFA, AFA):
 
         .. attention:: object is modified in place"""
         l = copy(self.Initial)
-        s0 = self.StateName(l.pop())
+        s0 = self.stateIndex(l.pop())
         while l:
             s = self.stateIndex(l.pop())
             self.mergeStates(s0, s)
@@ -885,6 +1323,7 @@ def genRandomTrie(maxL, Sigma, safe=True):
     # noinspection PyUnboundLocalVariable
     def _descend(s1, ens, safe1, m):
         sons = 0
+        final = None
         if not safe1:
             final = random.randint(0, 1)
         if safe1:
@@ -982,6 +1421,7 @@ def genRndTriePrefix(maxL, Sigma, ClosedP=False, safe=True):
                     trie.addFinal(s1)
                     return True
 
+    ensurance = None
     if safe:
         ensurance = _genEnsurance(maxL, Sigma)
     trie = ADFA()
@@ -1034,3 +1474,4 @@ def stringToADFA(s):
             new.addFinal(st)
     new.setInitial(len(s) - 1)
     return new
+
