@@ -32,7 +32,7 @@ import subprocess
 from yappy_parser import Yappy, grules
 
 from fa import NFA, DFA
-from fl import DCFA
+from fl import DFCA
 import common
 
 
@@ -138,7 +138,7 @@ class ParserGrail(Yappy):
                     break
         if isDeterministic:
             if "l" in self.eq.keys():
-                fa = DCFA()
+                fa = DFCA()
                 fa.setLength = self.eq["l"]
             else:
                 fa = DFA()
