@@ -28,6 +28,7 @@ ICDFA Random generation binding
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
 
 import random
+import time
 import common
 import fa
 
@@ -38,15 +39,23 @@ class ICDFArgen(object):
     :var int n: number of states
     :var int k: size of the alphabet
     :var int pn: how more problable shall a non defined transition be?
+    :var int seed: seed for the random generator. Default is to generate a time & system dependent.
 
     .. seealso:: Marco Almeida, Nelma Moreira, and Rogério Reis. Enumeration and generation with a string automata
-       representation. Theoretical Computer Science, 387(2):93-102, 2007"""
+       representation. Theoretical Computer Science, 387(2):93-102, 2007
 
-    def __init__(self, n, k, nd=False, pn=1):
+    .. versionchanged:: 1.3.4 seed added to the random generator"""
+
+    def __init__(self, n, k, nd=False, pn=1, seed=0):
         self.N = dict()
         self.n = n
         self.k = k
+        if seed != 0:
+            self.seed = seed
+        else:
+            self.seed = hash(time.clock())
         foo = n ** k
+        random.seed(self.seed)
         if not nd:
             self.pn = 0
         else:
@@ -91,7 +100,7 @@ class ICDFArgen(object):
             else:
                 r -= foo
                 bar *= m
-        return m * k - 1  # todo check the need of this return
+        return m * k - 1
 
     def _rndT(self, i):
         r = random.randint(0, i + self.pn - 1)
@@ -121,37 +130,44 @@ class ICDFArnd(ICDFArgen):
 
     This is the class for the uniform random generator for Initially Connected DFAs
 
-    .. note::
-        This is an abstract class, not to be used directly"""
+    :var int n: number of states
+    :var int k: size of alphabet
+    :var int seed: seed for the random generator (if 0 uses time as seed)
 
-    def __init__(self, n, k):
-        super(ICDFArnd, self).__init__(n, k)
+    .. note::
+        This is an abstract class, not to be used directly
+
+    .. versionchanged:: 1.3.4 seed added to the random generator"""
+
+    def __init__(self, n, k, seed=0):
+        super(ICDFArnd, self).__init__(n, k, False, 1, seed)
 
     def __str__(self):
-        return "ICDFArnd %d %d" % (self.n, self.k)
+        return "ICDFArnd %d %d %d" % (self.n, self.k, self.seed)
 
 
 class ICDFArndIncomplete(ICDFArgen):
     """ Incomplete ICDFA random generator class
 
-    :var n: number of states
-    :type n: integer
-    :var k: size of alphabet
-    :type k: integer
-    :var bias: how often must the gost sink state appear (default None)
-    :type bias: float
+    :var int n: number of states
+    :var int k: size of alphabet
+    :var float bias: how often must the gost sink state appear (default None)
+    :var int seed: seed for the random generator (if 0 uses time as seed)
 
-    :raises IllegalBias: if a bias >=1 or <=0 is provided"""
 
-    def __init__(self, n, k, bias=None):
+    :raises IllegalBias: if a bias >=1 or <=0 is provided
+
+    .. versionchanged:: 1.3.4 seed added to the random generator"""
+
+    def __init__(self, n, k, bias=None, seed=0):
         if bias is None:
-            super(ICDFArndIncomplete, self).__init__(n, k, True, 1)
+            super(ICDFArndIncomplete, self).__init__(n, k, True, 1, seed)
         elif bias <= 0 or bias >= 1:
             raise common.IllegalBias()
         else:
             m = int((bias * n) / (1 - bias))
-            super(ICDFArndIncomplete, self).__init__(n, k, True, m)
+            super(ICDFArndIncomplete, self).__init__(n, k, True, m, seed)
         self.n, self.k, self.bias = n, k, bias
 
     def __str__(self):
-        return "ICDFArndIncomplete %d %d %d" % (self.n, self.k, self.bias)
\ No newline at end of file
+        return "ICDFArndIncomplete %d %d %d %d" % (self.n, self.k, self.bias, self.seed)
\ No newline at end of file
