@@ -5,7 +5,7 @@ Transducer manipulation.
 
 .. versionadded:: 1.0
 
-.. *Authors:* Rogério Reis & Nelma Moreira
+.. *Authors:* Rogério Reis, Nelma Moreira & Stavros Konstantinidis
 
 .. *This is part of FAdo project*   http://fado.dcc.fc.up.pt.
 
@@ -90,7 +90,11 @@ class Transducer(fa.NFA):
 
     def succintTransitions(self):
         """ Collects the transition information in a concat way suitable for graphical representation.
-        :rtype: list of tupples"""
+        :rtype: list of tupples
+
+        .. note:
+            tupples in the list are stateout, label, statein
+        """
         foo = dict()
         for s in self.delta:
             for c in self.delta[s]:
@@ -105,7 +109,7 @@ class Transducer(fa.NFA):
             s = "{0:s}/{1:s}".format(graphvizTranslate(str(cs[0][0])), graphvizTranslate(str(cs[0][1])))
             for c in cs[1:]:
                 s += ", {0:s}/{1:s}".format(graphvizTranslate(str(c[0])), graphvizTranslate(str(c[1])))
-            l.append((str(self.States[k[0]]), str(self.States[k[1]]), s))
+            l.append((str(self.States[k[0]]), s, str(self.States[k[1]])))
         return l
 
     def setOutput(self, listOfSymbols):
@@ -205,6 +209,52 @@ class GFT(Transducer):
                             new.addTransition(cst, z[n - 1][0], z[n - 1][1], lst)
         return new
 
+    def listOfTransitions(self):
+        """ Collects into a sorted list the transitions of the transducer.
+
+        :rtype: set of tuples"""
+        def _comp(v1, v2):
+            (v11, v12, v13, v14) = v1
+            (v21, v22, v23, v24) = v2
+            if v11 < v21:
+                return -1
+            if v11 > v21:
+                return 1
+            if v12 < v22:
+                return -1
+            if v12 > v22:
+                return 1
+            if v13 < v23:
+                return -1
+            if v13 > v23:
+                return 1
+            return 0
+
+        trList = []
+        for s in self.delta:
+            for c in self.delta[s]:
+                for (oc, s1) in self.delta[s][c]:
+                    trList.append((s, c, oc, s1))
+        trList.sort(_comp)
+        return trList
+
+    def codeOfTransducer(self):
+        """ Appends into one string the codes of the alphabets and initial and final
+        state sets and the set of transitions
+
+        :rtype: tuple"""
+
+        def _codeOfSet(S):
+            """ Collects into a sorted list the elements of the set S and then
+                returns the string representation of the list. The set S normally
+                consists of integers or strings"""
+            L = [x for x in S]
+            L.sort()
+            return str(L)
+
+        return 'GFT', _codeOfSet(self.Sigma) + _codeOfSet(self.Output) + _codeOfSet(self.Initial) + \
+               _codeOfSet(self.Final) + str(self.listOfTransitions())
+
 
 class SFT(GFT):
     """Standard Form Tranducer
@@ -273,10 +323,13 @@ class SFT(GFT):
         if sti in self.Final:
             self.Final.remove(sti)
         self._deleteRefInitial(sti)
+        toAdd = set()
+        toDel = set()
         for s in self.Final:
             if sti < s:
-                self.Final.remove(s)
-                self.Final.add(s - 1)
+                toDel.add(s)
+                toAdd.add(s-1)
+        self.Final = self.Final - toDel | toAdd
         for j in xrange(sti + 1, len(self.States)):
             if j in self.delta:
                 self.delta[j - 1] = self.delta[j]
@@ -369,6 +422,22 @@ class SFT(GFT):
         i = self.stateIndex(dest, True)
         self.addTransition(src, sym, out, i)
 
+    def addTransitionProductQ(self, src, dest, ddest, sym, out, futQ, pastQ):
+        """Add transition to the new transducer instance.
+
+        Version for the optimized product
+
+        :param src: source state
+        :param dest: destination state
+        :param ddest: destination as tuple
+        :param sym: symbol
+        :param out: output
+        :param set futQ: queue for later
+        :param set pastQ: past queue"""
+        if ddest not in pastQ:
+            futQ.add(ddest)
+        self.addTransition(src, sym, out, dest)
+
     def outputS(self, s):
         """Output label coming out of the state i
 
@@ -547,6 +616,10 @@ class SFT(GFT):
     def inIntersection(self, other):
         """ Conjunction of transducer and automata: X & Y.
 
+        .. note:: This is a fast version of the method that does not produce meaningfull state names.
+
+        .. note:: The resulting transducer is not trim.
+
         :param DFA|NFA other: the automata needs to be operated.
         :rtype: SFT"""
         if isinstance(other, fa.DFA):
@@ -567,15 +640,81 @@ class SFT(GFT):
         else:
             par2 = nother
         new = par1.productInput(par2)
-        for x in [(par1.States[a], par2.States[b]) for a in par1.Final for b in par2.Final]:
-            if x in new.States:
-                new.addFinal(new.stateIndex(x))
+        sz2 = len(par2.States)
+        for s1 in par1.Final:
+            for s2 in par2.Final:
+                new.addFinal(s1*sz2+s2)
         return new
 
     def productInput(self, other):
         """Returns a transducer (skeleton) resulting from the execution of the transducer with the automaton as
         filter on the input.
 
+        .. note:: This version does not use stateIndex() with the price of generating some unreachable sates
+
+        :param NFA other: the automaton used as filter
+        :rtype: SFT
+
+        .. versionchanged:: 1.3.3"""
+        new = SFT()
+        new.setSigma(self.Sigma.union(other.Sigma))
+        notDone = set()
+        done = set()
+        sz2 = len(other.States)
+        for _ in range(len(self.States) * sz2):
+            new.addState()
+        for s1 in self.Initial:
+            for s2 in other.Initial:
+                new.addInitial(s1*sz2+s2)
+                notDone.add((s1, s2))
+        while notDone:
+            state = notDone.pop()
+            done.add(state)
+            (s1, s2) = state
+            sti = s1*sz2+s2
+            k1, k2 = self.inputS(s1), other.inputS(s2)
+            for k in k1.intersection(k2):
+                for (symo, o1) in self.delta[s1][k]:
+                    for o2 in other.delta[s2][k]:
+                        new.addTransitionProductQ(sti, o1*sz2+o2, (o1, o2), k, symo, notDone, done)
+        return new
+
+    def inIntersectionSlow(self, other):
+        """ Conjunction of transducer and automata: X & Y.
+
+        .. note:: This is the slow version of the method that keeps meaningfull names of states.
+
+        :param DFA|NFA other: the automata needs to be operated.
+        :rtype: SFT"""
+        if isinstance(other, fa.DFA):
+            nother = other.toNFA().renameStates()
+        elif isinstance(other, fa.NFA):
+            nother = other.renameStates()
+        else:
+            raise common.FAdoGeneralError("Incompatible objects")
+        et, en = self.epsilonP(), nother.epsilonP()
+        if en:
+            par1 = self.dup()
+            par1.addEpsilonLoops()
+        else:
+            par1 = self
+        if et:
+            par2 = nother.dup()
+            par2.addEpsilonLoops()
+        else:
+            par2 = nother
+        new = par1.productInputSlow(par2)
+        for x in [(par1.States[a], par2.States[b]) for a in par1.Final for b in par2.Final]:
+            if x in new.States:
+                new.addFinal(new.stateIndex(x))
+        return new
+
+    def productInputSlow(self, other):
+        """Returns a transducer (skeleton) resulting from the execution of the transducer with the automaton as
+        filter on the input.
+
+        .. note:: This is the slow version of the method that keeps meaningfull names of states.
+
         :param NFA other: the automaton used as filter
         :rtype: SFT"""
         new = SFT()
@@ -640,7 +779,7 @@ class SFT(GFT):
                 for (so1, o1) in par1.delta[i1][s]:
                     if so1 in K:
                         for (so2, o2) in par2.delta[i2][so1]:
-                            new.addTransitionQ(i, (par1.States[o1], par1.States[o2]), s, so2, notDone, done)
+                            new.addTransitionQ(i, (par1.States[o1], par2.States[o2]), s, so2, notDone, done)
         for x in [(par1.States[a], par2.States[b]) for a in par1.Final for b in par2.Final]:
             if x in new.States:
                 new.addFinal(new.stateIndex(x))
@@ -825,6 +964,17 @@ class SFT(GFT):
         for i in range(len(self.States)):
             self.addTransition(i, Epsilon, Epsilon, i)
 
+    def evalWordSlowP(self, wp):
+        """Tests whether the transducer returns the second word using the first one as input
+
+        Note: original
+        :param tuple wp: pair of words
+        :rtype: bool"""
+        import fl
+        (win, wout) = wp
+        inT = self.inIntersectionSlow(fl.FL([win]).trieFA().toNFA())
+        return not inT.outIntersection(fl.FL([wout]).trieFA().toNFA()).emptyP()
+
     def evalWordP(self, wp):
         """Tests whether the transducer returns the second word using the first one as input
 
@@ -832,8 +982,8 @@ class SFT(GFT):
         :rtype: bool"""
         import fl
         (win, wout) = wp
-        inT = self.inIntersection(fl.FL([win]).trieFA().toNFA())
-        return not inT.outIntersection(fl.FL([wout]).trieFA().toNFA()).emptyP()
+        inT = self.inIntersection(fl.FL([win]).MADFA())
+        return not inT.outIntersection(fl.FL([wout]).MADFA()).emptyP()
 
     def emptyP(self):
         """Tests if the relation realized  the empty transducer
