@@ -32,8 +32,42 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
 
-import fa
 from comboperations import *
+from reex import *
+from fa import DFA
+
+# Useful automata
+
+
+def emptyDFA(sigma=None):
+    """
+    Returns the minimal DFA for emptyset (incomplete)
+
+    :param sigma:
+    :return:
+    """
+    d = DFA()
+    if sigma is not None:
+        d.setSigma(sigma)
+    i = d.addState()
+    d.setInitial(i)
+    return d
+
+
+def epsilonDFA(sigma=None):
+    """
+    Returns the minimal DFA for {epsilon} (incomplete)
+
+    :param sigma:
+    :return:
+    """
+    d = DFA()
+    if sigma is not None:
+        d.setSigma(sigma)
+    i = d.addState()
+    d.setInitial(i)
+    d.addFinal(i)
+    return d
 
 ### Worst case automata for each operation
 
@@ -52,6 +86,7 @@ witnessDFA = {"toDFA": [("toDFAWCMF", "int"),
               "conjunction": [("interWC", "int", "int")],
               "__or__": [("disjWC", "int", "int")],
               "shuffle": [("shuffleWC", "int", "int")],
+              "suff": [("suffWCe","int"),("suffWCd","int")],
               "starDisj": [("starDisjWC", "int", "int")],
               "starInter": [("starInterBC", "int", "int")],
               "disjWStar": [("disjWStarWC", "int", "int")]}
@@ -59,13 +94,14 @@ witnessDFA = {"toDFA": [("toDFAWCMF", "int"),
 
 def toDFAWC2MF(m=5):
     """ Worst case automata for toDFA(NFA) with n > 2, k=2
-    @see: A. R. Meyer and M. J. Fischer. Economy of description by automata,
+    ..seealso::
+    A. R. Meyer and M. J. Fischer. Economy of description by automata,
     grammars, and formal systems. Twelfth Annual Symposium on
     Switching and Automata Theory, 1971,  188–191. IEEE Society Press.
 
-    :arg m: number of states
+    :param m: number of states
     :type m: integer
-    :returns: a dfa
+    :return: a dfa
     :rtype: DFA"""
 
     if m < 3:
@@ -85,7 +121,7 @@ def toDFAWC2MF(m=5):
 
 def toDFAWC2(m=5):
     """ Worst case automata for toDFA(NFA) with n > 2, k=2
-    @see: F.R. Moore. On the bounds for state-set size in the proofs
+    ..seealso:: F.R. Moore. On the bounds for state-set size in the proofs
     of equivalence between deterministic, nondeterministic, and
     two-way finite automata. IEEE Transactions on computers, 2:1211–1214, 1971.
 
@@ -113,7 +149,7 @@ def toDFAWC2(m=5):
 
 def toDFAWC3(m=5):
     """ Worst case automata for toDFA(NFA) with n > 2, k=3.
-    @see: O. B. Lupanov. A comparison of two types of finite sources.
+    ..seealso:: O. B. Lupanov. A comparison of two types of finite sources.
     Problemy Kibernetiki, 9:321–326, 1963.
     
     :arg m: number of states
@@ -147,7 +183,7 @@ def toDFAWC3(m=5):
 def reversalWC3M(m=5):
     """ Worst case automata for reversal(DFA) with m > 2, k=3.
 
-    @see: Boris G. Mirkin. On dual automata. Kibernetika, 2:7–10, 1966.
+    ..seealso:: Boris G. Mirkin. On dual automata. Kibernetika, 2:7–10, 1966.
    
     :arg m: number of states
     :type m: integer
@@ -173,7 +209,7 @@ def starSC(m=5):
 
 def starWC(m=5):
     """ Worst case automata for star(DFA) with m > 2, k=2
-    @see: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
+    ..seealso:: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
     of some basic operations on regular languages.
     Theor. Comput. Sci., 125(2):315–328, 1994.
 
@@ -201,7 +237,7 @@ def starWC(m=5):
 def starWCM(m=5):
     """ Worst case automata for star(DFA) with m > 2, k=2
 
-    @see: A. N. Maslov. Estimates of the number of states of
+    ..seealso:: A. N. Maslov. Estimates of the number of states of
     finite automata. Dokllady Akademii Nauk SSSR, 194:1266–1268, 1970. 
     
     :arg m: number of states
@@ -243,7 +279,7 @@ def concatSC(m, n, k=1):
 def concatWCM(m=4, n=4):
     """ Worst case automata for catenation(DFA,DFA) with m,n > 1, k=2,
 
-    @see: A. N. Maslov. Estimates of the number of states of
+    ..seealso:: A. N. Maslov. Estimates of the number of states of
     finite automata. Dokllady Akademii Nauk SSSR, 194:1266–1268, 1970. 
     :arg m: number of states
     :arg n: number of states
@@ -281,7 +317,7 @@ def concatWCM(m=4, n=4):
 
 def concatWC(m=6, n=6):
     """ Worst case automata for catenation(DFA,DFA) with m,n > 1
-    @see: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
+    ..seealso:: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
     of some basic operations on regular languages.
     Theor. Comput. Sci., 125(2):315–328, 1994.
     :arg m: number of states
@@ -311,11 +347,51 @@ def concatWC(m=6, n=6):
         d2.addTransition(i, "c", 1)
     return d1, d2
 
+def concatWCB(m=4, n=4):
+    """ Worst case automata for catenation(DFA,DFA) with m,n > 1, k=2,
+
+    ..seealso::Jirásek, J., Jiráaskováa, G., Szabari, A., 2005.
+     State complexity of concatenation and complementation of regular
+     languages. Int. J. Found. Comput. Sci. 16 (3), 511–529.
+    :arg m: number of states
+    :arg n: number of states
+    :type m: integer
+    :type n: integer
+    :returns: two dfas
+    :rtype: (DFA, DFA)"""
+
+    if n < 2 or m < 2:
+        raise TestsError("number of states must be both greater than 1")
+    d1, d2 = DFA(), DFA()
+    d1.setSigma(["a", "b"])
+    d1.States = range(m)
+    d1.setInitial(0)
+    d1.addFinal(m - 1)
+    d1.addTransition(m - 1, "b", 0)
+    d1.addTransition(m - 1, "a", m - 1)
+    for i in range(m - 1):
+        d1.addTransition(i, "a", i)
+        d1.addTransition(i, "b", i + 1)
+    d2.setSigma(["a", "b"])
+    d2.States = range(n)
+    d2.setInitial(0)
+    d2.addFinal(n - 1)
+    d2.addTransition(n - 1, "a", 0)
+    d2.addTransition(n - 1, "b", 0)
+    d2.addTransition(0, "a", 0)
+    d2.addTransition(0, "b", 1)
+    d2.addTransition(n - 2, "a", n - 1)
+    for i in range(1,n - 1):
+        d2.addTransition(i, "a", i + 1)
+        d2.addTransition(i, "b", i + 1)
+
+    return d1, d2
+
 
 def interWC(m=6, n=5):
     """ Worst case automata for intersection(DFA,DFA) with m,n >1
 
-    @see: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
+    ..seealso:: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
     of some basic operations on regular languages.
     Theor. Comput. Sci., 125(2):315–328, 1994.
     :arg m: number of states
@@ -346,7 +422,7 @@ def interWC(m=6, n=5):
 
 def disjWC(m=6, n=5):
     """ Worst case automata for disjunction(DFA,DFA) with m,n >1
-    @see: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
+    ..seealso:: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
     of some basic operations on regular languages.
     Theor. Comput. Sci., 125(2):315–328, 1994.
     :arg m: number of states
@@ -383,7 +459,7 @@ def disjWC(m=6, n=5):
 def reversalMB(m=8):
     """Worst case automata for reversal(DFA)
 
-    @see: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
+    ..seealso:: S. Yu, Q. Zhuang, and K. Salomaa. The state complexities
     of some basic operations on regular languages.
     Theor. Comput. Sci., 125(2):315–328, 1994.
     :arg m: number of states
@@ -413,7 +489,7 @@ def reversalMB(m=8):
 def reversalWC3L(m=5):
     """ Worst case automata for reversal(DFA) with m > 2, k=3
 
-    @see: E. L. Leiss. Succinct representation of regular languages
+    ..seealso:: E. L. Leiss. Succinct representation of regular languages
         by boolean automata ii. Theor. Comput. Sci., 38:133–136, 1985.
     :arg m: number of states
     :type m: integer
@@ -469,7 +545,7 @@ def reversalternaryWC(m=5):
 
 def reversalbinaryWC(m=5):
     """Worst case automata for reversal(DFA) binary
-    @see: G. Jir{\'a}skov{\'a} and J. S\v ebej. Note on Reversal of binary regular languages. Proc. DCFS 2011, 
+    ..seealso:: G. Jir{\'a}skov{\'a} and J. S\v ebej. Note on Reversal of binary regular languages. Proc. DCFS 2011,
     LNCS 6808, Springer, pp 212-221.
     @arg m: number of states
     @type m: integer
@@ -507,9 +583,12 @@ def reversalbinaryWC(m=5):
 
 def shuffleWC(m=3, n=3):
     """Worst case automata for shuffle(DFA,DFA) with m.n>1
-    @see:C. Campeanu, K. Salomaa, and S. Yu. Tight lower bound for
+
+    ..seealso::
+     C. Campeanu, K. Salomaa, and S. Yu. Tight lower bound for
     the state complexity of shuffle of regular languages.
     Journal of Automata, Languages and Combinatorics, 7(3):303–310, 2002.
+
     :arg m: number of states
     :arg n: number of states
     :type m: integer
@@ -546,7 +625,8 @@ def shuffleWC(m=3, n=3):
 
 def starDisjWC(m=6, n=5):
     """Worst case automata for starDisj(DFA,DFA) with m.n>1
-     @see: Arto Salomaa, Kai Salomaa, and Sheng Yu. 'State complexity of
+
+     ..seealso: Arto Salomaa, Kai Salomaa, and Sheng Yu. 'State complexity of
     combined operations'. Theor. Comput. Sci., 383(2-3):140–152, 2007.
     :arg m: number of states
     :arg n: number of states
@@ -583,7 +663,7 @@ def starDisjWC(m=6, n=5):
 
 def starInterBC(m=3, n=3):
     """Bad case automata for starInter(DFA,DFA) with m,n>1
-    @see: Arto Salomaa, Kai Salomaa, and Sheng Yu. 'State complexity of
+    ..seealso:: Arto Salomaa, Kai Salomaa, and Sheng Yu. 'State complexity of
     combined operations'. Theor. Comput. Sci., 383(2-3):140–152, 2007.
     :arg m: number of states
     :arg n: number of states
@@ -628,7 +708,7 @@ def starInterBC(m=3, n=3):
 
 def disjWStarWC(m=6, n=5):
     """
-     @see: Yuan Gao and Sheng Yu. 'State complexity of union and intersection
+     ..seealso:: Yuan Gao and Sheng Yu. 'State complexity of union and intersection
   combined with star and reversal'. CoRR, abs/1006.3755, 2010.
   :arg m: number of states
   :arg n: number of states
@@ -671,7 +751,7 @@ def disjWStarWC(m=6, n=5):
 def unionWCTk2(m=6, n=6):
     """ @ worst-case family union where
     @m>=2 and n>=2 and k=2
-    @see: Gao, Y., Salomaa, K., Yu, S.: Transition complexity of
+    ..seealso:: Gao, Y., Salomaa, K., Yu, S.: Transition complexity of
     incomplete dfas. Fundam. Inform.  110(1-4), 143–158 (2011)
     @ the conjecture in this article fails for this family
     :arg m: number of states
@@ -920,30 +1000,33 @@ def starWCT1(m=5):
     return f
 
 
-def universal(n, l=["a", "b", "c"], Finals=None, dialect=False, d=None):
+def universal(n, l= None, Finals=None, dialect=False, d=None):
     """Universal witness for state compelxity
-    :arg n: number of states
-    :type n: integer
-    :arg l: alphabet
-    :type l: list of strings
-    :arg Finals: list of final states
-    :type Finals: list of integers
+    :arg int n: number of states
+    :arg [str] l: alphabet
+    :arg [int] Finals: list of final states
+    :arg bool dialect: is it a dialect
     :returns: dfa 
     :rtype: DFA
     """
+    if n < 3:
+        raise TestsError("number of states must be greater than 2")
     u = DFA()
     u.States = range(n)
-    u.setSigma(l)
+    if l is None:
+        l = ("a", "b", "c")
+    u.setSigma(list(l))
     u.setInitial(0)
     u.addFinal(n - 1)
     u.addTransition(0, "b", 1)
     u.addTransition(1, "b", 0)
-    u.addTransition(n - 1, "c", 0)
+    if "c" in l:
+        u.addTransition(n - 1, "c", 0)
     for i in range(n):
         u.addTransition(i, "a", (i + 1) % n)
         if i >= 2:
             u.addTransition(i, "b", i)
-        if i != n - 1:
+        if i != n - 1 and "c" in l:
             u.addTransition(i, "c", i)
     return u
 
@@ -962,6 +1045,7 @@ def boundarySC(n, k):
     return 4 ** (n - 1) - nCr(n - 1, k - 1) + 2 ** (n - k) * 2 ** (n - 1) - 3 ** (n - k) * 2 ** (k - 1) + 2 ** (
         k - 1) * 2 ** (n - 1) - 3 ** (k - 1) * 2 ** (n - k) + 1
 
+# Don't care automata
 
 def dcMilano1(n):
     """Return the special dcNFA to prove the titness of proposed bound
@@ -1016,3 +1100,256 @@ def dcMilano2(n):
         new.addTransition(st[s + 2 * n], 'b', st[s])
     new.addFinal(st[n - 1])
     return new
+
+# Closure operations
+
+def suffWCe(m=3):
+    """Witness for suff(L) when L does not have empty as a quotient
+
+     :rtype: DFA
+
+     ..seealso:
+          Janusz A. Brzozowski, Galina Jirásková, Chenglong Zou,
+          Quotient Complexity of Closed Languages.
+          Theory Comput. Syst. 54(2): 277-292 (2014)
+
+     """
+    if m< 3:
+        raise TestsError("number of states must be greater than 2")
+    f = DFA()
+    f.setSigma(["a", "b"])
+    f.States = range(m)
+    f.setInitial(0)
+    f.addFinal(0)
+    f.addTransition(0, "a", 1)
+    f.addTransition(1, "a", 2)
+    f.addTransition(0, "b", 0)
+    f.addTransition(1, "b", 0)
+    for i in range(2, m):
+        f.addTransition(i, "a", (i + 1) % m)
+        f.addTransition(i, "b", i)
+    return f
+
+def suffWCd(m=3):
+
+    """Witness for suff(L) when L has  empty as a quotient
+
+    :rtype: DFA
+
+    ..seealso: as above
+    """
+    if m< 3:
+        raise TestsError("number of states must be greater than 2")
+    f = DFA()
+    f.setSigma(["a", "b"])
+    f.States = range(m)
+    f.setInitial(0)
+    f.addFinal(0)
+    f.addTransition(0, "a", 1)
+    f.addTransition(1, "a", 2)
+    f.addTransition(0, "b", m-1)
+    f.addTransition(1, "b", 0)
+    f.addTransition(m-1, "b", m-1)
+    f.addTransition(m-1, "a", m-1)
+    for i in range(2, m-1):
+        f.addTransition(i, "a", (i + 1) % (m-1))
+        f.addTransition(i, "b", i)
+    return f
+
+def suffWCsynt(m=3):
+  """ Worst case witness for synt of suff(L)
+
+  """
+  if m< 3:
+     raise TestsError("number of states must be greater than 2")
+  f = DFA()
+  f.setSigma(["a", "b", "c", "d", "e"])
+  f.States = range(m)
+  f.setInitial(0)
+  f.addFinal(m-1)
+  f.addTransition(0, "a", 0)
+  f.addTransition(0, "b", 0)
+  f.addTransition(0, "c", 0)
+  f.addTransition(0, "d", 0)
+  f.addTransition(0, "e", 1)
+  f.addTransition(1, "a", 2)
+  f.addTransition(1, "b", 2)
+  f.addTransition(1, "c", 1)
+  f.addTransition(1, "d", 1)
+  f.addTransition(1, "e", 1)
+  f.addTransition(2, "b", 1)
+  f.addTransition(2, "e", 1)
+  f.addTransition(2, "c", 2)
+  f.addTransition(2, "d", 2)
+  f.addTransition(2, "a", 3)
+  for i in range(3,m-1):
+    f.addTransition(i, "a", (i+1)% m)
+    f.addTransition(i,"b",i)
+    f.addTransition(i,"c",i)
+    f.addTransition(i,"d",i)
+    f.addTransition(i,"e",1)
+  f.addTransition(m-1,"a",1)
+  f.addTransition(m-1,"c",1)
+  f.addTransition(m-1,"e",1)
+  f.addTransition(m-1,"d",0)
+  return f
+
+def booleanWCSymGrp(m=3):
+  """Witness for symmetric group
+
+   :rtype: DFA
+   ..seealso:
+   	Jason Bell, Janusz A. Brzozowski, Nelma Moreira, Rogério Reis.
+   	Symmetric Groups and Quotient Complexity of Boolean Operations.
+   	ICALP (2) 2014: 1-12
+  """
+  if m< 3:
+     raise TestsError("number of states must be greater than 2")
+  f = DFA()
+  f.setSigma(["a", "b"])
+  f.States = range(m)
+  f.setInitial(0)
+  f.addFinal(0)
+  f.addFinal(1)
+  f.addTransition(0, "a", 1)
+  f.addTransition(1, "a", 0)
+  f.addTransition(0, "b", 1)
+  f.addTransition(1, "b", 2)
+  for i in range(2, m):
+    f.addTransition(i, "b", (i + 1) % m)
+    f.addTransition(i, "a", i)
+  return f
+
+def suffFreeSyntWC(m=5):
+    """
+
+    """
+    if m < 5:
+     raise TestsError("number of states must be greater than 2")
+    f = DFA()
+    f.setSigma(["a", "b", "c","d", "e"])
+    f.States = range(m)
+    f.setInitial(0)
+    f.addFinal(1)
+    f.addTransition(0,"a",m-1)
+    f.addTransition(0,'b',m-1)
+    f.addTransition(0,'c',m-1)
+    f.addTransition(0,'d',m-1)
+    f.addTransition(0,'e',1)
+    f.addTransition(1,"a",2)
+    f.addTransition(1,'c',1)
+    f.addTransition(1,'e',m-1)
+    f.addTransition(1,'d',m-1)
+    f.addTransition(1,'b',2)
+    f.addTransition(2,'b',1)
+    f.addTransition(2,"a",3)
+    f.addTransition(2,"c",2)
+    f.addTransition(2,"e",m-1)
+    f.addTransition(2,'d',2)
+    f.addTransition(1,'d',m-1)
+    f.addTransition(m-2,'c',1)
+    f.addTransition(m-2,'a',1)
+    for sym in f.Sigma:
+        f.addTransition(m-1,sym,m-1)
+    for i in range(3,m-1):
+        f.addTransition(i,"b",i)
+        if i != m-2:
+            f.addTransition(i,"c",i)
+            f.addTransition(i,"a",(i+1))
+        f.addTransition(i,"d",i)
+        f.addTransition(i,"e",m-1)
+    return f
+
+
+
+
+#shuffle
+
+def shuffle_ReBC(n=3):
+    """ Withess for Shuffle toDFA
+    """
+
+    if n < 2:
+        raise TestsError("at least n=2")
+    rs = star(concat(atom('a0'), atom('b0')))
+    for i in range(1, n):
+        rs = shuffle(rs,star(concat(atom('a'+str(i)),atom('b'+str(i)))))
+    return rs
+
+def shuffle_ReBCF(n=3):
+    """
+    :param n:
+    :return:
+    """
+    if n<2:
+        raise TestsError("at least n=2")
+    rs = concat(atom('a0'),atom('b0'))
+    for i in range(1,n):
+        rs = shuffle(rs,concat(atom('a'+str(i)),atom('b'+str(i))))
+    return rs
+
+def shuffle_NFABC(n=3):
+    """
+
+    :param  int n:
+    :rtype: atom
+    """
+    if n<2:
+        raise TestsError("at least n=2")
+    ns = star(concat(atom('a0'), atom('b0'))).toNFA()
+    for i in range(1, n):
+        ni = star(concat(atom('a'+str(i)), atom('b'+str(i)))).toNFA()
+        ns = ns.shuffle(ni)
+    return ns
+
+
+def shuffle_ReWC(n=4):
+    """
+
+    :param int n:
+    :rtype: atom
+    """
+    if n<2:
+        raise TestsError("at least n=2")
+    rs = atom('a0')
+    for i in range(1, n):
+        rs = shuffle(rs, atom('a'+str(i)))
+    return rs
+
+
+def intersectionReWC(n, sym = 'a'):
+    if n == 0:
+        return epsilon()
+    if n == 1:
+        return star(atom(sym))
+    else:
+        s1 = set()
+        s2 = set()
+        for i in range(1,n+1):
+            ri = str2xre(sym * i)
+            s1.add(star(ri))
+            s2.add(ri)
+        r1 = sdisj(frozenset(s1))
+        r2 = sstar(sdisj(frozenset(s2)))
+        return sconj(frozenset([r1,r2]))
+
+
+def revFibonnacci(n):
+    a = DFA ()
+    a.setSigma (["a", "b"]),
+    for i in range (n):
+        a.addState (i)
+    a.setInitial (0)
+    for i in range (n):
+        if i % 2 != 0:
+            if i < n - 1:
+                a.addTransition (i, 'b', i + 1)
+            if i < n - 2:
+                a.addTransition (i, 'a', i + 2)
+        else:
+            if i < n - 2:
+                a.addTransition (i, 'b', i + 2)
+            if i < n - 1:
+                a.addTransition (i, 'a', i + 1)
+    a.addFinal (n - 1)
+    return a
\ No newline at end of file
