@@ -45,19 +45,6 @@ class xre(reex.regexp):
         """Representation of the regular expression's syntactical tree."""
         return 'xre({0:>s})'.format(self.__str__())
 
-    def unionSigma(self, reg):
-        """Returns the union of two alphabets
-
-        :param reg: xre
-        :rtype: set
-        """
-        if self.Sigma is None:
-            return reg.Sigma
-        elif reg.Sigma is None:
-            return self.Sigma
-        else:
-            return self.Sigma.union(reg.Sigma)
-
     def derivative(self, sigma):
         """Derivative of the regular expression in relation to the given symbol.
 
@@ -199,11 +186,6 @@ class xre(reex.regexp):
             Fundam. Inform. 45(3), 195-205 (2001)"""
         return {xepsilon(self.Sigma)}
 
-    def toDFA(self, dfa_method="dfaDerivatives"):
-        """DFA that accepts the regular expression's language.
-        :param dfa_method: """
-        return self.__getattribute__(dfa_method)()
-
     def toNFA(self, nfa_method="nfaPD"):
         """NFA that accepts the regular expression's language.
         :param nfa_method: """
@@ -401,6 +383,7 @@ class xspecialConst(xre):
     def __init__(self, sigma=None):
         """
         :param sigma: """
+        super (xspecialConst, self).__init__ (sigma)
         self.Sigma = sigma
 
     def __copy__(self):
@@ -1718,7 +1701,7 @@ def _disjrpn(arg1, arg2):
     if len(s) == 1:
         return s.pop()
     else:
-        return xdisj(s,self.Sigma)
+        return xdisj(s, arg1.Sigma)
 
 
 def _conjrpn(arg1, arg2):
@@ -1951,7 +1934,7 @@ def to_x(reg):
   
     :param reg:
     :arg re: the FAdo representation regexp for a regular expression.
-    :type reg: regexp
+    :type reg: reex.regexp
     :rtype: xre"""
     if type(reg) is reex.regexp:
         return to_xregexp(reg)
