@@ -49,7 +49,7 @@ _Version = "1.9.4"
 NIL = ""
 
 
-class Lexer:
+class Lexer(object):
     """Class for lexical analyser to use with the parser
 
         :var rules: lexical rules
@@ -286,7 +286,7 @@ class TableError(YappyError):
         return "%s" % self.value
 
 
-class CFGrammar:
+class CFGrammar(object):
     """ Class for context-free grammars
 
         :var rules: grammar rules
@@ -636,40 +636,40 @@ class CFGrammar:
         while e:
             e = 0
             for s in self.nonterminals:
-                for i in self.ntr[s]:
-                    r = self.rules[i][1]
-                    if r == []:
+                for i1 in self.ntr[s]:
+                    r = self.rules[i1][1]
+                    if not r:
                         continue
-                    for i in range(len(r)):
-                        if r[i] in self.terminals:
-                            if i < len(r) - 1:
-                                if self.derive_ter.has_key(r[i + 1]):
+                    for i1 in range(len(r)):
+                        if r[i1] in self.terminals:
+                            if i1 < len(r) - 1:
+                                if self.derive_ter.has_key(r[i1 + 1]):
                                     if not self.derive_ter.has_key(s):
                                         self.derive_ter[s] = Set([])
-                                    if self.derive_ter[s].s_append(r[i]):
+                                    if self.derive_ter[s].s_append(r[i1]):
                                         e = 1
                                 break
                             else:
                                 if not self.derive_ter.has_key(s):
                                     self.derive_ter[s] = Set([])
-                                if self.derive_ter[s].s_append(r[i]):
+                                if self.derive_ter[s].s_append(r[i1]):
                                     e = 1
 
                                 break
                         else:
                             """ non-terminal"""
-                            if self.derive_ter.has_key(r[i]):
+                            if self.derive_ter.has_key(r[i1]):
                                 if not self.derive_ter.has_key(s):
                                     self.derive_ter[s] = Set([])
-                                if self.derive_ter[s].s_extend(self.derive_ter[r[i]]) == 1:
+                                if self.derive_ter[s].s_extend(self.derive_ter[r[i1]]) == 1:
                                     e = 1
-                            if i > 0 and self.nullable[r[i]]:
+                            if i1 > 0 and self.nullable[r[i1]]:
                                 continue
                             else:
                                 break
 
 
-class LRtable:
+class LRtable(object):
     """Class for construction of a *LR* table
 
         :var gr: a context-free grammar
@@ -962,7 +962,7 @@ class LR1table(LRtable):
             for i in c:
                 for s in symbols:
                     valid = self.goto(i, s)
-                    if valid != []:
+                    if valid:
                         if c.s_append(valid): e = 1
         return c
 
@@ -1048,8 +1048,7 @@ class LALRtable1(LRtable):
         list of terminals: is coded as a dictonary with key
         C{(rule_number,dot_position)} and value a set of terminals
         """
-        i0 = {}
-        i0[(len(self.gr.rules) - 1, 0)] = Set([self.gr.endmark])
+        i0 = {(len (self.gr.rules) - 1, 0): Set ([self.gr.endmark])}
         c = Set([self.closure(i0)])
         symbols = self.gr.terminals + self.gr.nonterminals
         e = 1
@@ -1196,7 +1195,7 @@ class LALRtable(LALRtable1):
                                 r = self.AfterDotTer(item, c[i], p)
                                 if self.gr.ntr.has_key(A):
                                     for k in self.gr.ntr[A]:
-                                        if self.gr.rules[k][1] == []:
+                                        if not self.gr.rules[k][1]:
                                             for a in r:
                                                 self.add_action(i, a, 'reduce', k)
 
@@ -1225,8 +1224,7 @@ class LALRtable(LALRtable1):
         of terminals: is coded as a dictionary with key 
         C{(rule_number,dot_position)} and value a set of terminals.
         """
-        i0 = {}
-        i0[(len(self.gr.rules) - 1, 0)] = Set([self.gr.endmark])
+        i0 = {(len (self.gr.rules) - 1, 0): Set ([self.gr.endmark])}
         c = Set([i0])
         symbols = self.gr.terminals + self.gr.nonterminals
         """ kernel LR(0) items """
@@ -1250,8 +1248,7 @@ class LALRtable(LALRtable1):
             lh[nk] = {}  #Set([])
             for (n, i) in k.keys():
                 lh[nk][(n, i)] = Set([])
-                j = {}
-                j[(n, i)] = Set([(self.gr.dummy)])
+                j = {(n, i): Set ([(self.gr.dummy)])}
                 j = self.closure(j)
                 for s in symbols:
                     for (m1, j1) in j.keys():
@@ -1325,7 +1322,7 @@ class LALRtable(LALRtable1):
             return f
 
 
-class LogLR:
+class LogLR(object):
     """Class for LR table construction report:
         @ivar expect: number of shit/reduce conflicts expected
         @type expect: integer
@@ -1347,7 +1344,7 @@ class LogLR:
             self.conflicts[type] = [(i, a, value1, value2)]
 
 
-class LRparser:
+class LRparser(object):
     """Class for LR parser
 
        @ivar cfgr: context free grammar 
@@ -1401,7 +1398,8 @@ class LRparser:
         if 'nosemrules' in args:
             self.nosemrules = args['nosemrules']
         else:
-            self.nosemrules = 0
+            self.nosemrules = None
+
         db = whichdb.whichdb(table_shelve)
         if not (db == None or db == "" or no_table == 0):
             try:
@@ -1674,7 +1672,7 @@ class LRparser:
         return gr
 
 
-class LRBuildparser:
+class LRBuildparser(object):
     """Class for LR parser: without shelve and semantic rules(obsolete)
     """
 
@@ -1801,7 +1799,7 @@ class Yappy(LRparser):
         if args.has_key('usrdir') and os.path.isdir(args['usrdir']):
             table = string.rstrip(args['usrdir']) + '/' + table
         if os.path.dirname(table) == "" or os.path.exists(os.path.dirname(table)):
-            LRparser.__init__(self, grammar, table, no_table, tabletype, operators, noconflicts, expect, **args)
+            LRparser.__init__(self, grammar,table, no_table, tabletype, operators, noconflicts, expect, **args)
         else:
             sys.stderr.write("Directory %s do not exist\n" % table)
             sys.exit()
@@ -1981,7 +1979,7 @@ class Yappy_grammar(Yappy):
         lhs = arg[0]
 
         def grule(self, l):
-            if l == []: return (lhs, [], EmptySemRule)
+            if not l: return (lhs, [], EmptySemRule)
             if type(l[1]) is TupleType:
                 return (lhs, l[0], eval(l[1][0], globals(), context['locals']), l[1][1])
             else:
@@ -2015,11 +2013,12 @@ class Yappy_grammar(Yappy):
         reg -> ;
         a -> reg | reg ;
         """
-        self.input(st, {'locals': locals()})
+        d = self.input(st, {'locals': locals()})
+
         return self.context['rules']
 
 
-class Stack:
+class Stack(object):
     """ A simple class to implement stacks"""
 
     def __init__(self, start=[]):
